(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit AppTranslator;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Buttons, Menus, ExtCtrls;

const CCaption= 1; CHint=2;
      CAlreadyTranslated=16;
type TMyString = class(TObject)
        public
          X : string;
          constructor Create(T:string);
     end;


type
  TAppTranslator = class(TComponent)
  private
    { private-Deklarationen }
    AvailLangs: TStringList;
    TransList : TStringList;
    MDIList   : TStringList;

    function WriteToFile(Name:string):boolean;
    function ReadFromFile(Name:string):boolean;
    procedure AddTransString(Ident, Translation:string);
    procedure ClearStringList(X:TStringList);
  protected
    { protected-Deklarationen }
  public
    { public-Deklarationen }
    LangName  : string;
    FileName  : string;
    procedure Translate;
    procedure TranslateMDI(X:TComponent);
    function GetStr(X:string):string;
    destructor Free;
    procedure TranslateComponent(Where:TComponent; Which, What:string);
    procedure ChangeComponent(X:TComponent; Where:integer; What:string);
    procedure AddTransList(Where: TComponent; MDI:boolean);
    procedure ScanForLanguages(Path:string; Add:boolean);
    procedure GetAvailLanguages(X:TStrings);
    function GetLangFileName(X:string):string;
  published
    { published-Deklarationen }
  end;

procedure Register;

implementation

constructor TMyString.Create(T:string);
begin
  X:=T;
end;


procedure TAppTranslator.AddTransList(Where: TComponent; MDI:boolean);
var F:TextFile;

  procedure HandleComponent(X:TComponent; Path:string);

    procedure WriteCaptionHint(Path, Caption, Hint:string);
    begin
      if Caption<>'' then begin
        if not MDI then WriteLn(F, '!['+Path+'.Caption]')
                   else WriteLn(F, '?['+Path+'.Caption]');
        WriteLn(F, Caption);
      end;
      if Hint<>'' then begin
        if not MDI then WriteLn(F, '!['+Path+'.Hint]')
                   else WriteLn(F, '?['+Path+'.Hint]');
        WriteLn(F, Hint);
      end;
    end;

  begin
    Path:=Path+X.Name;
    if (X.Tag and CAlreadyTranslated)=0 then begin
      if X is TForm then WriteCaptionHint(Path, TForm(X).Caption, TForm(X).Hint);
      if X is TGroupBox then WriteCaptionHint(Path, TGroupBox(X).Caption, TGroupBox(X).Hint);
      if X is TTabSheet then WriteCaptionHint(Path, TTabSheet(X).Caption, TTabSheet(X).Hint);
      if X is TButton then WriteCaptionHint(Path, TButton(X).Caption, TButton(X).Hint);
      if X is TMenuItem then WriteCaptionHint(Path, TMenuItem(X).Caption, TMenuItem(X).Hint);
      if X is TLabel then WriteCaptionHint(Path, TLabel(X).Caption, TLabel(X).Hint);
      if X is TPanel then WriteCaptionHint(Path, TPanel(X).Caption, TPanel(X).Hint);
      if X is TTreeView then WriteCaptionHint(Path, '', TTreeView(X).Hint);
      if X is TBevel then WriteCaptionHint(Path, '', TBevel(X).Hint);
      if X is TEdit then WriteCaptionHint(Path, '', TEdit(X).Hint);
      if X is TComboBox then WriteCaptionHint(Path, '', TComboBox(X).Hint);
      if X is TMemo then WriteCaptionHint(Path, '', TMemo(X).Hint);
      if X is TCheckBox then WriteCaptionHint(Path, TCheckBox(X).Caption, TCheckBox(X).Hint);
      if X is TRadioButton then WriteCaptionHint(Path, TRadioButton(X).Caption, TRadioButton(X).Hint);
      if X is TSpeedButton then WriteCaptionHint(Path, TSpeedButton(X).Caption, TSpeedButton(X).Hint);
//      if X is TOpenDialog then WriteCaptionHint(Path, TOpenDialog(X).Title, '');
    end;
    X.Tag:=X.Tag or CAlreadyTranslated;
  end;

  procedure CheckAllComponents(X:TComponent; Path:string; Level:integer);
  var i:integer;
  begin
    for i:=0 TO X.ComponentCount-1 do HandleComponent(X.Components[i], Path);
  end;

var i:integer;
begin
  AssignFile(F, FileName);
  Append(F);
  CheckAllComponents(Where, '',0); // Check the Forms, too
  for i:=0 TO Where.ComponentCount-1 do begin
    CheckAllComponents(Where.Components[i], Where.Components[i].Name+'.',0);
  end;
  CloseFile(F);
end;

procedure TAppTranslator.ScanForLanguages(Path:string;  Add:boolean);
var SearchRec : TSearchRec;
    Result    : integer;
    F         : TextFile;
    S         : string;
    i         : integer;
begin
  if not Assigned(AvailLangs) then begin
    AvailLangs:=TStringList.Create;
    AvailLangs.Duplicates:=dupIgnore;
    AvailLangs.Sorted:=True;
  end;
  if not Add then begin
    for i:=AvailLangs.Count-1 downto 0 do begin
      AvailLangs.Objects[i].Free;
      AvailLangs.Delete(i);
    end;
  end;
  if Path[Length(Path)]<>'\' then Path:=Path+'\';
  Result := FindFirst(Path+'*.Texts', faAnyFile, SearchRec);
  while Result = 0 do begin
    try
      AssignFile(F, Path+SearchRec.Name);
      Reset(F);
      try
        ReadLn(F, S);
      except S:='' end;
      if Pos('LANGUAGE=',AnsiUpperCase(S))>0 then begin
        Delete(S, 1, Pos('LANGUAGE=',AnsiUpperCase(S))+Length('LANGUAGE=')-1);
        AvailLangs.AddObject(S, TMyString.Create(Path+SearchRec.Name));
      end;
    finally
      CloseFile(F);
      Result := FindNext(SearchRec);
    end;
  end;
  FindClose(SearchRec);
end;

procedure TAppTranslator.GetAvailLanguages(X:TStrings);
begin
  if Assigned(X) and Assigned(AvailLangs) then X.AddStrings(AvailLangs);
end;

function TAppTranslator.GetLangFileName(X:string):string;
var Nr:integer;
begin
  Result:='';
  if Assigned(AvailLangs) and AvailLangs.Find(X, Nr) then
    Result:=TMyString(AvailLangs.Objects[Nr]).X;
end;

procedure TAppTranslator.TranslateMDI(X:TComponent);
var i: integer;
begin
  for i:=0 TO MDIList.Count-1 do
    TranslateComponent(X, MDIList[i], TMyString(MDIList.Objects[i]).X);
end;

procedure TAppTranslator.Translate;
begin
  if not Assigned(TransList) then TransList:=TStringList.Create;
  TransList.Duplicates:=dupIgnore;
  TransList.Sorted:=True;

  if not Assigned(MDIList) then MDIList:=TStringList.Create;
  MDIList.Duplicates:=dupIgnore;
  MDIList.Sorted:=True;
  ReadFromFile(FileName);
end;


procedure TAppTranslator.ChangeComponent(X:TComponent; Where:integer; What:string);
begin
  case Where of
    CCaption: begin
                if X is TForm then TForm(X).Caption:=What;
                if X is TGroupBox then TGroupBox(X).Caption:=What;
                if X is TTabSheet then TTabSheet(X).Caption:=What;
                if X is TButton then TButton(X).Caption:=What;
                if X is TMenuItem then TMenuItem(X).Caption:=What;
                if X is TLabel then TLabel(X).Caption:=What;
                if X is TPanel then TPanel(X).Caption:=What;
                if X is TCheckBox then TCheckBox(X).Caption:=What;
                if X is TRadioButton then TRadioButton(X).Caption:=What;
                if X is TSpeedButton then TSpeedButton(X).Caption:=What;
    //            if X is TOpenDialog then TOpenDialog(X).Title:=What;
              end;
    CHint   : begin
                if X is TForm then TForm(X).Hint:=What;
                if X is TGroupBox then TGroupBox(X).Hint:=What;
                if X is TTabSheet then TTabSheet(X).Hint:=What;
                if X is TButton then TButton(X).Hint:=What;
                if X is TMenuItem then TMenuItem(X).Hint:=What;
                if X is TLabel then TLabel(X).Hint:=What;
                if X is TPanel then TPanel(X).Hint:=What;
                if X is TTreeView then TTreeView(X).Hint:=What;
                if X is TBevel then TBevel(X).Hint:=What;
                if X is TEdit then TEdit(X).Hint:=What;
                if X is TComboBox then TComboBox(X).Hint:=What;
                if X is TMemo then TMemo(X).Hint:=What;
                if X is TCheckBox then TCheckBox(X).Hint:=What;
                if X is TRadioButton then TRadioButton(X).Hint:=What;
                if X is TSpeedButton then TSpeedButton(X).Hint:=What;
              end;
    end;
    X.Tag:=X.Tag or CAlreadyTranslated;
end;

procedure TAppTranslator.ClearStringList(X:TStringList);
var i:integer;
begin
  for i:=X.Count-1 downto 0 do begin
    TMyString(X.Objects[i]).Free;
    X.Delete(i);
  end;
end;

function TAppTranslator.GetStr(X:string):string;
var Nr:integer;
begin
  if not Assigned(TransList) then begin
    Result:=X;
    Exit;
  end;
  if TransList.Find(X, Nr) then Result:=TMyString(TransList.Objects[Nr]).X else Result:=X;
end;

procedure TAppTranslator.AddTransString(Ident, Translation:string);
begin
  TransList.AddObject(Ident, TMyString.Create(Translation));
end;

procedure TAppTranslator.TranslateComponent(Where:TComponent;Which, What:string);
var F,C:TComponent;
begin
  F:=Where.FindComponent(Copy(Which, 1, Pos('.', Which)-1));
  if Assigned(F) then begin
    Delete(Which, 1, Pos('.', Which));
    C:=F.FindComponent(Copy(Which, 1, Pos('.', Which)-1));
    if not Assigned(C) then begin
        C:=F;
        if Pos('.', Which)>0 then begin
          Application.MessageBox(PChar(Which), 'Unused Key', mb_OK);
          Exit;
        end;
    end;
    Delete(Which, 1, Pos('.', Which));
    if Assigned(C) then begin
      if AnsiCompareText(Trim(Which), 'Caption')=0 then ChangeComponent(C, CCaption, What)
       else
        if AnsiCompareText(Trim(Which), 'Hint')=0 then ChangeComponent(C, CHint, What);
    end;
  end;
end;

function TAppTranslator.ReadFromFile(Name:string):boolean;
var Txt     : TextFile;
    Key     : string;
    Content : string;
    S       : string;
begin
  ClearStringList(TransList);
  ClearStringList(MDIList);
  Result:=True;
  try
    AssignFile(Txt, Name);
    try
      Reset(Txt);
      if EoF(Txt) then raise Exception.Create('Translation error');
      ReadLn(Txt, S);
      if Pos('LANGUAGE=',AnsiUpperCase(S))>0 then begin
        Delete(S, 1, Pos('LANGUAGE=',AnsiUpperCase(S))+Length('LANGUAGE=')-1);
        LangName:=S;
      end else raise Exception.Create('not a LANGUAGE File');
      while not EoF(Txt) do begin
        repeat
          ReadLn(Txt, Key);
          Key:=Trim(Key);
        until ((Length(Key)>2) and (Key[1]<>';')) or (EoF(Txt));
// if Length (Key) = 0 and we try TO read --> explode
// no matter What is in the following string: it is the Content, even if it is
// marked as a comment
        if EoF(Txt) then raise Exception.Create('Translation error');
        ReadLn(Txt, Content);
        if (Length(Key)>2) then begin
          if (Length(Key)>4) and (Key[1]='!') then begin
            Delete(Key, 1, 2);
            Delete(Key, Length(Key), 1);
            TranslateComponent(Application, Key, Content)
          end else if (Length(Key)>4) and (Key[1]='?') then begin
            Delete(Key, 1, 2);
            Delete(Key, Length(Key), 1);
            MDIList.AddObject(Key, TMyString.Create(Content));
           end else AddTransString(Key, Content);
//          AddTransString(Key, Content);
        end;
      end;
    finally
      CloseFile(Txt);
    end;
  except
    Result:=FALSE;
  end;
end;

function TAppTranslator.WriteToFile(Name:string):boolean;
begin
  Result:=True;
end;

destructor TAppTranslator.Free;
var i:integer;
begin
  ClearStringList(TransList);
  TransList.Free;
  if Assigned(AvailLangs) then begin
    for i:=AvailLangs.Count-1 downto 0 do begin
      TMyString(AvailLangs.Objects[i]).Free;
      AvailLangs.Delete(i);
    end;
    AvailLangs.Free;
  end;
end;

procedure Register;
begin
  RegisterComponents('Enigon', [TAppTranslator]);
end;

end.
