(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit Modules;

interface

USES Classes, Windows, SysUtils, Elemente,
     ImportConstants, ImportTypes, RailyTypes;

type TModule = class(TObject)
                  Elements : TStringList;
                  Info     : TModuleInfo;
                  Changed  : boolean;
                  Factor   : Double;
                  constructor Create;
                  procedure ImportWR1(Name:string);
{$IFDEF XMAS}     procedure ExportWR1(Name:string); {$ENDIF}
                  function Save(Name:string):boolean;
                  function Load(Name:string):boolean;
                  function GetElement(ElName:string; var X:TElement):boolean;
                  procedure GetAllOfTyp(Typ:TImportSchienenTyp; X:TStringList);
                  procedure RemoveElement(ElName:string);
                  procedure AddElement(ElName:string; var X:TElement);
                  destructor Free;
               end;

type TModuleCollection = class(TObject)
                  Modules         : TStringList;
                  AvailableModules: TStringList;
                  RemarkList      : TStringList;
                  constructor Create;
                  procedure ScanForModules(Path:string);

                  procedure ManufacturerList(X:TStringList);
                  function ManufacturerOf(ModName:string):string;
                  function SpurweiteOf(ModName:string):Double;
                  function SpurweiteOfSystem(System:string):Double;
                  procedure UpdateShortInfo(ModName:string;Sp:Double);
                  procedure AddModule(FileName:string;M:TModule);

                  procedure LoadModules(Path:string);
                  function LoadModule(ModName:string):boolean;
                  function IsAvailable(ModName:string):boolean;
                  function IsLoaded(ModName:string):boolean;
                  function KnownFileName(ModName:string):string;
                  function GetModule(ModName:string; var X:TModule):boolean;
                  function SaveModule(ModName:string):boolean;
                  function IsSaved(ModName:string):boolean;
                  procedure GetAllOfAType(ML:TStringList; Typ:TImportSchienenTyp; var X:TStringList);
                  function ConvertModule(Name, NewPath:string; var X:TModule; var NewName:string; var Error:integer): boolean;

                  procedure ClearRemarkList;
                  procedure ReadRemarks(Path, Ext:string);
                  procedure WriteRemarks(Path, Ext:string);

                  destructor Free;
               end;


const ModuleFileDesc:string='This is a RAILY 2.0 module file.'+#10+#13+
                     'RAILY 2.0 (C) 1994-1997 by Thomas M. Frey'+#10+#13+
                     'Use RAILY to view or edit This file.'+#10+#13+#$1a;

      ModuleSign = 'RFW2.0MDL';


implementation


//******************************************************************************
// Definition of TModule
//******************************************************************************
constructor TModule.Create;
begin
  Factor:=1;
  Elements:=TStringList.Create;
  Elements.Sorted:=True;
  StrPCopy(Info.Description, ModuleFileDesc);
  StrPCopy(Info.Signature, ModuleSign);
  Changed:=FALSE;
end;

procedure TModule.ImportWR1(Name:string);
var ImportedHeader  : TImportModuleRecord;
    Number          : integer;
    i               : integer;
    X               : TElement;
    S               : TMemoryStream;

// Eat eats a ObjType from a TStreamRec (BP7.0)
  procedure Eat(w:WORD);
  var Dummy:WORD;
  begin
    S.Read(Dummy, SizeOF(WORD));
    {$IFDEF DEBUG} WriteLn('ate ', Dummy, ' vgl ', w);{$ENDIF}
    //if Dummy<>w then raise EFileCorrupt.Create('Illegal ID in procedure ImportWR1(var S:TStream)');
  end;

// GetInt reads an integer
  function GetOldInt:Int16; begin S.Read(Result, 2) end;

begin
  S:=TMemoryStream.Create;
  S.LoadFromFile(Name);
  try
    Eat(ImportModuleID);
    S.Read(ImportedHeader, SizeOF(TImportModuleRecord)); // why is SizeOF 2 bytes greater?
    // answer DELPHI USES aligned records

    // Read the essential of a collection
    Eat(ImportCollID); Number:=GetOldInt; Eat(ImportDummy);Eat(ImportDummy);

    WITH ImportedHeader do begin
      StrCopy(Info.Name, ModulName);
      StrCopy(Info.System, System);
      StrCopy(Info.Manufacturer, Hersteller);
      StrCopy(Info.Author, Autor);
      StrCopy(Info.Remark, Anmerkung);
    end;

    {$IFDEF DEBUG}
    WITH ImportedHeader do begin
      WriteLn('Reading ', SizeOF(TImportModuleRecord), ' bytes of header Info');
      WriteLn(ModulName);
      WriteLn(ModulDatei);
      WriteLn(System);
      WriteLn(Hersteller);
      WriteLn(Autor);
      WriteLn(Anmerkung);
      WriteLn('Number of elments to import:', Number);
      WriteLn('SizeOF Pointer:', SizeOF(Pointer));
    end;
    {$ENDIF}

    for i:=0 to Number-1 do begin
    {$IFDEF DEBUG}
      WriteLn('---------------------------------------------------');
      WriteLn('Element #',i);
    {$ENDIF}
      Eat(ImportElementID);
      X:=TElement.Create;
      X.ImportWR1(TStream(S));
      Elements.AddObject(X.Info.Name, X);
    {$IFDEF DEBUG}
      WriteLn('===================================================');
      WriteLn;

//      readln;
    {$ENDIF}
    end;
   except
         ON Exception do ;//WriteLn('an Error occured while importing');
   end;
   {$IFDEF DEBUG} WriteLn('Stream position at ', S.position,' of ', S.size, '.'); {$ENDIF}
   S.Free;
   Changed:=FALSE;
end;

{$IFDEF XMAS}
  procedure TModule.ExportWR1(Name:string);
  var ImportedHeader  : TImportModuleRecord;
      Number          : integer;
      i               : integer;
      X               : TElement;
      S               : TMemoryStream;

  // SpitOut eats a ObjType from a TStreamRec (BP7.0)
    procedure SpitOut(w:WORD);
    begin
      S.Write(w, SizeOF(WORD));
    end;

    procedure PutOldInt(X:Int16); begin S.Write(X, 2) end;

  begin
    S:=TMemoryStream.Create;
    try
      SpitOut(ImportModuleID);
      WITH ImportedHeader do begin
        StrCopy(ModulName, Info.Name);
        StrPCopy(ModulDatei, Copy(ExtractFileName(Name), 1, 8));
        StrCopy(System, Info.System);
        StrCopy(Hersteller, Info.Manufacturer);
        StrCopy(Autor, Info.Author);
        StrCopy(Anmerkung, Info.Remark);
      end;
      S.Write(ImportedHeader, SizeOF(TImportModuleRecord));

      Number:=Elements.Count;
      SpitOut(ImportCollID); PutOldInt(Number); SpitOut(Number);SpitOut(5);
      for i:=0 to Number-1 do begin
        SpitOut(ImportElementID);
        X:=TElement(Elements.Objects[i]);
//        WriteLn(X.Info.Name);
        X.ExportWR1(TStream(S));
      end;
     except
           //ON Exception do Warn(TransStr('[Error while exporting]');
     end;
     S.SaveToFile(Name);
     S.Free;
  end;
{$ENDIF}

function TModule.Save(Name:string):boolean;
var Number          : longint;
    i               : longint;
    S               : TMemoryStream;

begin
   Result:=True;
   try
     S:=TMemoryStream.Create;
     S.Write(Info, SizeOF(Info));
     Number:=Elements.Count;
     S.Write(Number, SizeOF(longint));
     for i:=0 to Number-1 do begin
       TElement(Elements.Objects[i]).ToStream(TStream(S));
     end;
     S.SaveToFile(Name);
   except
     Result:=FALSE;
   end;
   S.Free;
   Changed:=FALSE;
end;

function TModule.Load(Name:string):boolean;
var Number          : longint;
    i               : longint;
    X               : TElement;
    S               : TMemoryStream;
    Sys             : string;
begin
   Result:=True;
   try
     S:=TMemoryStream.Create;
     S.LoadFromFile(Name);
     S.Read(Info, SizeOF(Info));
     S.Read(Number, SizeOF(longint));
     if StrComp(Info.Signature, ModuleSign)=0 then begin
       for i:=0 to Number-1 do begin
         X:=TElement.Create;
         X.FromStream(TStream(S));
         Elements.AddObject(X.Info.Name, X);
       end;
     end;
     S.Free;
   except
     Result:=FALSE;
   end;
  Changed:=FALSE;
  Sys:=Copy(Info.System, 1, 2);
  if Sys = 'HO' then Factor:=87;
  if Sys = 'II' then Factor:=22;
  if Sys = 'i' then Factor:=32;
  if Sys = '1' then Factor:=32;
  if Sys = 'O' then Factor:=43.5;
  if Sys = 'S' then Factor:=64;
  if Sys = 'TT' then Factor:=120;
  if Sys = 'N' then Factor:=160;
  if Sys = 'Z' then Factor:=220;
  if Sys = 'G' then Factor:=32;
end;

function TModule.GetElement(ElName:string; var X:TElement):boolean;
var Nr:integer;
begin
  Result:=FALSE;
  if Elements.Find(ElName, Nr) then begin
    X:=TElement(Elements.Objects[Nr]);
    Result:=True;
  end;
end;

procedure TModule.GetAllOfTyp(Typ:TImportSchienenTyp; X:TStringList);
var i:integer;
    T:TElement;
begin
  for i:=0 to Elements.Count-1 do begin
    T:=TElement(Elements.Objects[i]);
    if Assigned(T) then if T.Info.TheType=Typ then X.AddObject(StrPas(T.Info.Name), TMyString.Create(StrPas(Info.Name)));
    // it is important to Free the associated Objects after Use
  end;
end;

procedure TModule.RemoveElement(ElName:string);
var Nr:integer;
begin
  if Elements.Find(ElName, Nr) then begin
    TElement(Elements.Objects[Nr]).Free;
    Elements.Delete(Nr);
  end;
  Changed:=True;
end;

procedure TModule.AddElement(ElName:string; var X:TElement);
begin
  Elements.AddObject(ElName, X);
  Changed:=True;
end;

destructor TModule.Free;
var i:integer;
begin
  for i:=Elements.Count-1 downto 0 do begin
    TElement(Elements.Objects[i]).Free;
    Elements.Delete(i);
  end;
  Elements.Free;
end;

//******************************************************************************
// Definition of TModuleCollection
//******************************************************************************

constructor TModuleCollection.Create;
begin
  Modules:=TStringList.Create;
  Modules.Sorted:=True;
  Modules.Duplicates:=DupIgnore;
  AvailableModules:=TStringList.Create;
  AvailableModules.Sorted:=True;
  AvailableModules.Duplicates:=DupIgnore;
  RemarkList:=TStringList.Create;
end;

function TModuleCollection.ConvertModule(Name, NewPath:string; var X:TModule; var NewName:string; var Error:integer): boolean;
var TX        : TModuleInfo;
    Dummy     : integer;
    R         : longint;
    F         : file;
begin
  Result:=True;
  Error:=0;
  try
    FileMode:=0;
    AssignFile(F, Name);
    Reset(F, 1);
    BlockRead(F, TX, SizeOF(TX), R);
    CloseFile(F);
    if (R<>SizeOF(TX)) or (TX.Signature<>ModuleSign) then begin
      X:=TModule.Create;
      X.ImportWR1(Name);
      if not Modules.Find(X.Info.Name, Dummy) then begin
        Modules.AddObject(X.Info.Name, X);
        if X.Info.Name='LGB' then X.Info.Manufacturer:='LGB';
        NewName:=NewPath+'RAILY_2_'+ExtractFileName(Name);
        X.Save(NewName);
      end else begin X.Free; Error:=1; end;
    end else begin Result:=FALSE; Error:=2; end;
  except
    Result:=FALSE;
    Error:=3;
  end;
  FileMode:=2;
end;

procedure TModuleCollection.LoadModules(Path:string);
var SearchRec : TSearchRec;
    Result    : integer;
    X         : TModule;
    TX        : TModuleInfo;
    Dummy     : integer;
//    Count     : integer;
    R         : longint;
    F         : file;
begin
//  Count:=0;
  Result := FindFirst(Path+'\*.mdl', faAnyFile, SearchRec);
  while Result = 0 do begin
    FileMode:=0;
    AssignFile(F, Path+'\'+SearchRec.Name);
    Reset(F, 1);
    BlockRead(F, TX, SizeOF(TX), R);
    CloseFile(F);
    if (R<>SizeOF(TX)) or (TX.Signature<>ModuleSign) then begin
//      WriteLn('Processing:', SearchRec.Name);
      X:=TModule.Create;
      X.ImportWR1(Path+'\'+SearchRec.Name);
      if not Modules.Find(X.Info.Name, Dummy) then begin
        Modules.AddObject(X.Info.Name, X);
        if X.Info.Name='LGB' then X.Info.Manufacturer:='LGB';
//        Count:=Count+X.Elements.Count;
        X.Save(Path+'\'+'RAILY_2_'+SearchRec.Name);
//        WriteLn('Converted');
      end else X.Free;
    end;
    Result := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
//  WriteLn('Finished, ', Count, ' Elements loaded.');
  FileMode:=2;
end;

procedure TModuleCollection.ScanForModules(Path:string);
var SearchRec : TSearchRec;
    Result    : integer;
    X         : TModuleInfo;
    P         : PShortModuleInfo;
    R         : integer;
    Dummy     : integer;
    F         : file;
begin
  if Path[Length(Path)]<>'\' then Path:=Path+'\';
  Result := FindFirst(Path+'*.mdl', faAnyFile, SearchRec);
  while Result = 0 do begin
    try
      FileMode:=0;
      AssignFile(F, Path+SearchRec.Name);
      Reset(F, 1);
      BlockRead(F, X, SizeOF(X), R);
      if R=SizeOF(X) then begin
        if X.Signature=ModuleSign then begin
          if not AvailableModules.Find(Path+SearchRec.Name, Dummy) then begin
            New(P);
            P^.FileName:=Path+SearchRec.Name;
            StrCopy(P^.Manufacturer, X.Manufacturer);
            StrCopy(P^.System, X.System);
            P^.Spurweite:=X.Spurweite;
            AvailableModules.AddObject(X.Name, TObject(P));
          end;
        end;
      end;
    finally
      CloseFile(F);
      Result := FindNext(SearchRec);
    end;
  end;
  FindClose(SearchRec);
  FileMode:=2;
end;

function TModuleCollection.LoadModule(ModName:string):boolean;
var X         : TModule;
begin
  if not IsLoaded(ModName) then begin
    if IsAvailable(ModName) then begin
      try
        X:=TModule.Create;
        Result:=X.Load(KnownFileName(ModName));
        Modules.AddObject(X.Info.Name, X);
      except
        Result:=FALSE;
      end;
    end else Result:=FALSE;
  end else Result:=True;
//  WriteLn('Modul:', ModName, ' hat ', X.Elements.Count);
end;

procedure TModuleCollection.AddModule(FileName:string;M:TModule);
var
    X         : TModuleInfo;
    P         : PShortModuleInfo;
begin
  if not Assigned(M) then Exit;
  Modules.AddObject(M.Info.Name, M);
  X:=M.Info;
  New(P);
  P^.FileName:=FileName;
  StrCopy(P^.Manufacturer, X.Manufacturer);
  StrCopy(P^.System, X.System);
  P^.Spurweite:=X.Spurweite;
  AvailableModules.AddObject(X.Name, TObject(P));
end;

function TModuleCollection.ManufacturerOf(ModName:string):string;
var Nr:integer;
begin
  if AvailableModules.Find(ModName, Nr) then begin
    Result:=Trim(StrPas(PShortModuleInfo(AvailableModules.Objects[Nr]).Manufacturer));
  end else Result:='';
end;

function TModuleCollection.SpurweiteOf(ModName:string):Double;
var Nr:integer;
begin
  if AvailableModules.Find(ModName, Nr) then begin
    Result:=PShortModuleInfo(AvailableModules.Objects[Nr]).Spurweite;
  end else Result:=0;
end;

procedure TModuleCollection.UpdateShortInfo(ModName:string;Sp:Double);
var Nr:integer;
begin
  if AvailableModules.Find(ModName, Nr) then begin
    PShortModuleInfo(AvailableModules.Objects[Nr]).Spurweite:=Sp;
  end;
end;

function TModuleCollection.SpurweiteOfSystem(System:string):Double;
var Nr : integer;
    w  : Double;
begin
  w:=0;
  for Nr:=0 to AvailableModules.Count-1 do begin
    if (StrPas(PShortModuleInfo(AvailableModules.Objects[Nr]).System)=System)
    and (PShortModuleInfo(AvailableModules.Objects[Nr]).Spurweite>w) then w:=PShortModuleInfo(AvailableModules.Objects[Nr]).Spurweite;
  end;
  Result:=w;
end;

procedure TModuleCollection.ManufacturerList(X:TStringList);
var Nr : integer;
begin
  if not Assigned(X) then Exit;
  X.Duplicates:=DupIgnore;
  X.Sorted:=True;
  for Nr:=0 to AvailableModules.Count-1 do begin
    X.Add(Trim(StrPas(PShortModuleInfo(AvailableModules.Objects[Nr]).Manufacturer)));
  end;
end;

function TModuleCollection.IsLoaded(ModName:string):boolean;
var D:integer;
begin
  Result:=Modules.Find(ModName, D);
end;

function TModuleCollection.IsAvailable(ModName:string):boolean;
var D:integer;
begin
  Result:=AvailableModules.Find(ModName, D);
end;

function TModuleCollection.KnownFileName(ModName:string):string;
var Nr:integer;
begin
  if AvailableModules.Find(ModName, Nr) then begin
    Result:=TrimLeft(TrimRight(PShortModuleInfo(AvailableModules.Objects[Nr]).FileName));
  end else Result:='';
end;

function TModuleCollection.SaveModule(ModName:string):boolean;
var F:file;
    M:TModule;
begin
  if IsLoaded(ModName) then begin
    try
      if FileExists(KnownFileName(ModName)) then begin
        AssignFile(F, KnownFileName(ModName));
        if FileExists(KnownFileName(ModName)+'.bak') then  DeleteFile (KnownFileName(ModName)+'.bak');
        Rename(F, KnownFileName(ModName)+'.bak');
      end;
    finally
      if GetModule(ModName, M) then begin
        Result:=M.Save(KnownFileName(ModName))
      end else Result:=FALSE;
    end;
  end else Result:=FALSE;
end;

function TModuleCollection.GetModule(ModName:string; var X:TModule):boolean;
var Nr:integer;
begin
  if Modules.Find(ModName, Nr) then begin
    X:=TModule(Modules.Objects[Nr]);
    Result:=True;
  end else Result:=FALSE;
end;

procedure TModuleCollection.GetAllOfAType(ML:TStringList; Typ:TImportSchienenTyp; var X:TStringList);
var i:integer;
    M:TModule;
begin
  for i:=0 to ML.Count-1 do if GetModule(ML.Strings[i], M) then begin
    M.GetAllOfTyp(Typ, X);
  end;
end;

function TModuleCollection.IsSaved(ModName:string):boolean;
var Nr:integer;
begin
  if Modules.Find(ModName, Nr) then begin
    Result:=not TModule(Modules.Objects[Nr]).Changed;
  end else Result:=FALSE;
end;

procedure TModuleCollection.ClearRemarkList;
var i:integer;
begin
  for i:=RemarkList.Count-1 downto 0 do begin
    TMyString(RemarkList.Objects[i]).Free;
    RemarkList.Delete(i);
  end;
end;

procedure TModuleCollection.ReadRemarks(Path, Ext:string);
var Name      : string;
    Where,
    What      : string;
    TF        : TextFile;
    i         : integer;
begin
  ClearRemarkList;
  for i:=0 to AvailableModules.Count-1 do begin
    Name:=PShortModuleInfo(AvailableModules.Objects[i]).FileName;
    Delete(Name, Length(Name)-3, 4);
    Name:=Name+'.'+Ext;
    if FileExists(Name) then begin
//    WriteLn('Consulting:',Name);
      AssignFile(TF, Name); Reset(TF);
      while not EoF(TF) do begin
        readln(TF, Where);
        while (Where='') and not EoF(TF) do readln(TF, Where);
        readln(TF, What);
//        WriteLn(Where,':',What);
        RemarkList.AddObject(Where, TMyString.Create(What));
      end;
      CloseFile(TF);
    end;
  end;
end;

procedure TModuleCollection.WriteRemarks(Path, Ext:string);
var i, j, Count:integer;
    Name:string;
    TF:TextFile;
    Inst:TElement;
    M:TModule;
begin
  for i:=0 to Modules.Count-1 do begin
    M:=NIL;
    if GetModule(Modules[i], M) then begin
      Name:=KnownFileName(Modules[i]);
      Delete(Name, Length(Name)-3, 4);
      Name:=Name+'.'+Ext;
      Count:=0;
      for j:=0 to M.Elements.Count-1 do begin
        Inst:=TElement(M.Elements.Objects[j]);
        if Assigned(Inst) then begin
          if string(Inst.Info.Remark)<>'' then begin
            Inc(Count);
            if Count=1 then begin
              AssignFile(TF, Name);
              Rewrite(TF);
            end;
            WriteLn(TF, '['+M.Info.Name+']'+Inst.Info.Name);
            WriteLn(TF, Inst.Info.Remark);
          end;
        end;
      end;
      if Count>0 then CloseFile(TF);
    end;
  end;
end;

destructor TModuleCollection.Free;
var i:integer;
begin
  for i:=Modules.Count-1 downto 0 do begin
    TModule(Modules.Objects[i]).Free;
    Modules.Delete(i);
  end;
  for i:=AvailableModules.Count-1 downto 0 do begin
    Dispose(PShortModuleInfo(AvailableModules.Objects[i]));
    AvailableModules.Delete(i);
  end;
  ClearRemarkList;
  RemarkList.Free;
end;

end.
