(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit NewModuleDialog;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Globals, Dialogs;

type
  TModulePropertyDLG = class(TForm)
    Bevel1: TBevel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    Label1: TLabel;
    NameEdit: TEdit;
    Label2: TLabel;
    FilenameEdit: TEdit;
    Label3: TLabel;
    SystemEdit: TEdit;
    Label4: TLabel;
    ManufacturerEdit: TComboBox;
    Label5: TLabel;
    AuthorEdit: TEdit;
    Label6: TLabel;
    RemarkMemo: TMemo;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Label7: TLabel;
    TrackWidthEdit: TEdit;
    ElWidthEdit: TEdit;
    Label8: TLabel;
    procedure FormActivate(Sender: TObject);
    procedure SystemEditExit(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure NameEditChange(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    procedure Reset;
  end;

var
  ModulePropertyDLG: TModulePropertyDLG;

implementation

{$R *.DFM}

procedure TModulePropertyDLG.FormActivate(Sender: TObject);
var X:TStringList;
begin
  X:=TStringList.Create;
  ManufacturerEdit.Items.Clear;
  GlobalModules.ManufacturerList(X);
  ManufacturerEdit.Items.AddStrings(X);
  X.Free;
end;

procedure TModulePropertyDLG.Reset;
begin
  TrackWidthEdit.Text := '';
  ElWidthEdit.Text := '';
  ManufacturerEdit.Text:='';
  NameEdit.Text:='';
  FilenameEdit.Text:='';
  SystemEdit.Text:='';
  AuthorEdit.Text:='';
  RemarkMemo.Text:='';
end;

procedure TModulePropertyDLG.SystemEditExit(Sender: TObject);
var Sp:Double;
    St:string;
begin
  if TrackWidthEdit.Text = '' then begin
    Sp:=GlobalModules.SpurweiteOfSystem(SystemEdit.Text);
    Str(Sp:2:2, St);
    TrackWidthEdit.Text:=Trim(St);
  end;
end;

{$HINTS OFF}
procedure TModulePropertyDLG.BitBtn1Click(Sender: TObject);
var Dummy, TW, EW:Double;
    c :integer;

begin
  NameEdit.Text:=Trim(NameEdit.Text);
  TrackWidthEdit.Text:=Trim(TrackWidthEdit.Text);

  ElWidthEdit.Text:=Trim(ElWidthEdit.Text);

  ManufacturerEdit.Text:=Trim(ManufacturerEdit.Text);

  if ManufacturerEdit.Text='' then begin
    ManufacturerEdit.SetFocus;
    Inform(TransStr('[Manufacturer required]'));
    Exit;
  end;

  // if a new module is created check whether this name was used before
  if NameEdit.Enabled then begin
    if GlobalModules.IsAvailable(NameEdit.Text) then begin
      NameEdit.SetFocus;
      Inform(TransStr('[name already in use]'));
      Exit;
    end;
  end;

	Val(TrackWidthEdit.Text, TW, c);
	if c<>0 then  begin
		TrackWidthEdit.SetFocus;
		Beep;
		Exit;
	end;

	Val(ElWidthEdit.Text, EW, c);
	if c<>0 then  begin
		ElWidthEdit.SetFocus;
		Beep;
		Exit;
	end;

	if EW<TW then begin
		Beep;
		Inform(TransStr('[EW must be >= TW]'));
		ElWidthEdit.SetFocus;
		Exit;
	end;

	ModalResult:=mrOk;
end;
{$HINTS ON}

procedure TModulePropertyDLG.NameEditChange(Sender: TObject);
begin
  FilenameEdit.Text:=GlobalModulePath+Trim(NameEdit.Text)+'.MDL';
  FilenameEdit.SelStart:=Length(FilenameEdit.Text);
end;

end.
