(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit Plan;

interface

USES Classes, Graphics, Windows, SysUtils, Controls, Forms, RailyTypes, Elemente, Modules, Globals,
     Instances, ImportTypes, ImportConstants, GeneralConstants, ClipBrd, Math,
     UInventory, UPlanUndo;

type TPlan = class (TObject)
               private
                 ActualSearch  : longint;

               public
                 Info             : TPlanInfo;
                 InstanceElements : TList;
                 Selected         : TBits;
                 UL, LR           : TDoublePoint;
                 SUL, SLR         : TDoublePoint; // Rectangle around selection
                                                  // has to be recalculated before use
                 DrawInfo         : TDrawInfo;
                 IsSaved          : boolean;
                 TablePolygon     : TList;
                 UserTexts        : TList;
                 Inventory        : TInventory;
                 UndoHelper       : TPlanUndo;
                 Layers           : TBits;
                 PlanDestroyed    : boolean;
                 OkNotTheSameModules: function (S1, S2:string; P:TDoublePoint):boolean of object;
                 OkNotTheSameSystems: function (S1, S2:string; P:TDoublePoint):boolean of object;
               private
                 procedure ClearPlan;

               public
                 constructor Create;
                 procedure CalcRectAround;
                 function Load(FName:string):boolean;
                 function Save(FName:string):boolean;
                 function ImportWR1(FName:string):boolean;
                 procedure Draw(X:TCanvas; Ax, Ay, fx, fy:Double; Overview, NoSelection:boolean;Exception:integer);
                 procedure SpeedDraw(X:TCanvas; Ax, Ay, fx, fy:Double; Overview, NoSelection:boolean; Exception:integer; aTL, aDR:TDoublePoint);
                 procedure UpdateSingle(Nr:integer; X:TCanvas; Ax, Ay, fx, fy:Double; Overview, Unselected:boolean);
                 procedure DrawTable(X:TCanvas; Ax, Ay, fx, fy:Double; Selected:integer; Numbers:boolean);
                 procedure DrawUserText(X:TCanvas; Ax, Ay, fx, fy:Double);

                 procedure ClearSelection;
                 procedure SetDrawInfo;
                 procedure CalcRectAroundSelection;
                 procedure CalcRectAroundInstList(CurrentInstList:TList; var TUL, TLR:TDoublePoint);

                 procedure SelectAll;
                 procedure Select(Nr:integer);
                 procedure Invert(Nr:integer);
                 procedure Unselect(Nr:integer);
                 procedure CopySelected;

                 procedure GetInstanceList(var X:TList);
                 function GetSelectedFreeCnctList(var Res:TList):boolean;
                 function GetAllFreeCnctList(var Res:TList):boolean;

                 procedure CreateCnctList(X: TList; var Res:TList);
                 procedure SetInstancesPosTo(List, Folge:TList; p1:TDoublePoint; a1:Double; p2:TDoublePoint; a2:Double);

                 function IsConnectible(I1: TInstance; CNr1: integer; I2: TInstance; CNr2: integer; DistTol, AngleTol:Double):boolean;
                 procedure AddInstance(Inst:TInstance; Selectem:boolean; Undoable, NewID:boolean; ResultList:TList);
                 procedure AddInstancesList(var X:TList; Selectem:boolean; ResultList:TList);

                 function DeleteSelected:integer;
                 procedure SelectedToMeta;
                 procedure AllToMeta;

                 procedure AddElement(Inst: TInstance; Selectit:boolean);
                 procedure DeleteElement(Nr:integer; Undoable, FreeInst:boolean);
                 function GetUniqueID:integer;
                 function GetElement(Nr:integer; var I:TInstance):boolean;
                 function FindElementWithID(Id:integer; var Inst:TInstance; var Nr:integer):boolean;
                 function Hit(ClX, ClY:longint; Ax, Ay, fx, fy:Double; var Nr:integer):boolean;
                 function IsHit(Nr:integer;ClX, ClY:longint; Ax, Ay, fx, fy:Double):boolean;
                 function InRect(Nr:integer; UL, DR:TDoublePoint):boolean;

                 function PointToGridPoint(X:TDoublePoint):TDoublePoint;

                 procedure ClearTablePolygon;
                 procedure BuildStandardTable;

                 procedure CalcInventory(All, Unused:boolean; List:TStringList);

                 procedure UndoSomething(X:TObject);
                 procedure Undo;

                 function OneSelectedTyp(var Typ:TImportSchienenTyp; var Flex:boolean):boolean;

                 procedure ShowAllLayers;
                 //user Texts
                 procedure TextsToStream(S:TStream);
                 procedure TextsFromStream(S:TStream);
                 procedure ClearUserTexts;
                 function CalcLength:Double;
                 function CalcRealLength:Double;

                 destructor Free;
             end;

const CNormalID  = 100;
      CFlexID    = 200;

implementation

const PlanFileDesc:string='This is a RAILY 2.1 Plan file.'+#10+#13+
                     'RAILY 2.0 (C) 1993-1998 by Thomas M. Frey'+#10+#13+
                     'use RAILY to view or edit This file.'+#10+#13+#$1a;

      PlanSign = 'RFW2.0PLN';

constructor TPlan.Create;
begin
  InstanceElements:=TList.Create;
  Selected:=TBits.Create;

  StrPCopy(Info.Description, PlanFileDesc);
  StrPCopy(Info.Signature, PlanSign);

  Info.Signature:=PlanSign;

  StrPCopy(Info.Description, PlanFileDesc);
  TablePolygon:=TList.Create;

  IsSaved:=TRUE;
  PlanDestroyed:=FALSE;

  Inventory:=TInventory.Create;
  UndoHelper:=TPlanUndo.Create;
  UndoHelper.UndoCallBack:=UndoSomething;

  DrawInfo.Blacked:=FALSE;
  DrawInfo.OverRuleColor:=FALSE;
  Layers:=TBits.Create;
  Layers.Size:=51;
  ShowAllLayers;
  UserTexts:=TList.Create;
end;

procedure TPlan.SetDrawInfo;
begin
//Set default DrawInfo
  DrawInfo.DoubleDraw:=0;
  if GlobalPlanSingle then DrawInfo.DoubleDraw:=DrawInfo.DoubleDraw or CDrawInner;
  if GlobalPlanDouble then DrawInfo.DoubleDraw:=DrawInfo.DoubleDraw or CDrawOuter;
  if GlobalPlanBettung then DrawInfo.DoubleDraw:=DrawInfo.DoubleDraw or CDrawBettung;
  if GlobalPlanPrec<16 then GlobalPlanPrec:=16;
  DrawInfo.Precision:=GlobalPlanPrec;
  DrawInfo.Exact:=GlobalPlanExact;
  DrawInfo.ShowRailNames:=GlobalPlanShowNames;
  DrawInfo.CnctDraw:=0;
  DrawInfo.Selected:=FALSE;
  DrawInfo.ForceTransparent:=GlobalPlanTransPoly;
  if GlobalPlanSeparate then DrawInfo.CnctDraw:=DrawInfo.CnctDraw or CDrawCnctSeparator;
  if GlobalPlanShowArrow then DrawInfo.CnctDraw:=DrawInfo.CnctDraw or CDrawCnctArrow;
end;

function TPlan.GetUniqueID:integer;
var I:integer;
    Inst:TInstance;
    X:TBits;
begin
  X:=TBits.Create;
  X.Size:=InstanceElements.Count + 1;
  for I:=0 to X.Size-1 do X[I]:=FALSE;
  for I:=0 to InstanceElements.Count-1 do
    if GetElement(I, Inst) and (Inst.Id>=0) and (Inst.Id<X.Size) then X[Inst.Id]:=TRUE;
  Result:=X.OpenBit;
  X.Free;
end;

procedure TPlan.CalcRectAround;
var I     : integer;
    Inst  : TInstance;
    a, B  : TSinglePoint;
    First : boolean;
    P     : TDoublePoint;
begin
  First:=TRUE;
  UL.X:=0; UL.y:=0; // Prevent div by zeros if something fails
  LR.X:=1; LR.y:=1;
  for I:=0 to InstanceElements.Count-1 do begin
    Inst:=TInstance(InstanceElements.Items[I]);
    if Assigned(Inst) then begin
      a:=Inst.UL; B:=Inst.LR;
      if First then begin UL.X:=a.X;UL.y:=a.y; LR.X:=B.X;LR.y:=B.y; First:=FALSE; end;
      if UL.X>a.X then UL.X:=a.X;
      if UL.y>a.y then UL.y:=a.y;
      if LR.X<B.X then LR.X:=B.X;
      if LR.y<B.y then LR.y:=B.y;
    end;
  end;
  if GlobalPlanIncTable then begin
    for I:=0 to TablePolygon.Count-1 do begin
      P:=TPointObject(TablePolygon[I]).P;
      if P.X<UL.X then UL.X:=P.X;
      if P.y<UL.y then UL.y:=P.y;
      if P.X>LR.X then LR.X:=P.X;
      if P.y>LR.y then LR.y:=P.y;
    end;
  end;
end;

procedure TPlan.AddElement(Inst: TInstance; Selectit:boolean);
var Nr:integer;
begin
  Nr:=InstanceElements.Add(Inst);

  Selected.Size:=InstanceElements.Count;
  // The last two conditions should always be TRUE ... anyway
  if Selectit and (Nr>=0) and (Nr<Selected.Size) then Selected[Nr]:=TRUE;
  IsSaved:=FALSE;
end;

procedure TPlan.DeleteElement(Nr:integer; Undoable, FreeInst:boolean);
var I, j:integer;
    Inst:TInstance;
    C, OC:TConnector;
    Other:TInstance;
    PosNr:integer;
    X    :TPrepAddInstance;
begin
// is it a valid Nr
  if (Nr>=0) and (Nr<InstanceElements.Count) then begin
    IsSaved:=FALSE;
    // Get "Inst" as The element to delete
    Inst:=InstanceElements[Nr];
    if Assigned(Inst) then begin // is Inst a vaild element ?
      //Undo ...
      if Undoable then begin
        X:=TPrepAddInstance.Create;
        Inst.ToStream(X.Backup);
        UndoHelper.AddState(X);
      end;
      // Check All The connected Elements
      for I:=0 to Inst.ConnectorCount-1 do begin
        if Inst.GetConnector(I, C) then begin
          if FindElementWithID(C.Nr, Other, PosNr) then begin
            // "Other" is an element connected to "Inst"
            if Assigned(Other) then begin
              // Check All its connectors
              for j:=0 to Other.ConnectorCount-1 do
                // reset those connected to "Inst"
                if Other.GetConnector(j, OC) and (OC.Nr=Inst.Id) then
                   Other.SetConnector(j, -1);
            end;
          end;
        end;
      end;

      // connectors are adjusted
      if FreeInst then Inst.Free;
      InstanceElements.delete(Nr);
      InstanceElements.Pack;
      Selected.Size:=InstanceElements.Count;
    end;
  end;
end;

function TPlan.Load(FName:string):boolean;
var C          : longint;
    S          : TMemoryStream;
    AnInst     : TInstance;
    I          : longint;
    TempInfo   : TPlanInfo;
    TDP        : TDoublePoint;
    Ok         : boolean;
begin
  Result:=TRUE;  // very optimistic
  Ok:=TRUE;
  ClearUserTexts;
  try
    S:=TMemoryStream.Create;
    try
      S.LoadFromFile(FName);
      S.Read(TempInfo, SizeOf(TempInfo));
      // is The file a RAILY for Windows 2.00 Plan ?
      if StrComp(PlanSign, TempInfo.Signature)=0 then begin
        Info:=TempInfo;
        S.Read(C, SizeOf(C));
        for I:=0 to C-1 do begin
          AnInst:=TInstance.Create;
          // Can The instance be created from stream ?
          if not AnInst.FromStream(TStream(S)) then begin
            AnInst.Free;
            Ok:=FALSE;
          end else AddElement(AnInst, FALSE);
          if S.Position>=S.Size then raise Exception.Create('FileError');
        end;
        if not Ok then begin
          Inform(TransStr('[ModuleError]'));
          PlanDestroyed:=TRUE;
        end;
        // Load The table
        ClearTablePolygon;
        S.Read(C, SizeOf(C));
        for I:=0 to C-1 do begin
          if S.Position>=S.Size then raise Exception.Create('FileError');
          S.Read(TDP, SizeOf(TDoublePoint));
          TablePolygon.Add(TPointObject.Create(TDP.X, TDP.y));
        end;

        // Load user Texts
        if S.Position>=S.Size then raise Exception.Create('FileError');
        TextsFromStream(S);

      end else Result:=FALSE;
    except
      // if anything goes wrong, everything is wrong...
      Result:=FALSE;
    end;
  finally
    S.Free;
  end;
  IsSaved:=Result;
end;

function TPlan.Save(FName:string):boolean;
var C:longint;
    S:TMemoryStream;
    I:longint;
begin
  InstanceElements.Pack;
  Result:=TRUE;
  try
    S:=TMemoryStream.Create;
    try
      S.Write(Info, SizeOf(Info));
      C:=InstanceElements.Count;
      S.Write(C, SizeOf(C));
      for I:=0 to C-1 do begin
        if not TInstance(InstanceElements.Items[I]).ToStream(TStream(S)) then begin
          Result:=FALSE;
        end;
      end;
      // Save The table
      C:=TablePolygon.Count;
      S.Write(C, SizeOf(C));
      for I:=0 to C-1 do begin
        S.Write(TPointObject(TablePolygon[I]).P, SizeOf(TDoublePoint));
      end;

      // Save user Texts
      TextsToStream(S);

      // Save The stream to The file
      S.SaveToFile(FName);
     except
       Result:=FALSE;
     end;
  finally
    S.Free;
  end;
  IsSaved:=TRUE;
  PlanDestroyed:=FALSE;
end;

function TPlan.ImportWR1(FName:string):boolean;
var TextInfo : TImportTextInfoRecord;
    GridInfo : TImportGridInfoRecord;
    Header   : TImportRLYHeader;
    S        : TMemoryStream;
    Number   : longint;
    I,j      : longint;
    AInstance: TInstance;
    X, y     : real;
    C        : char;
    TempStr  : string;
    T        : TUserTextEntry;
    Ok       : boolean;

  procedure Eat(w:WORD);
  var Dummy:WORD;
  begin
    S.Read(Dummy, SizeOf(WORD));
    {$IFDEF DEBUG} WriteLn('ate ', Dummy, ' vgl ', w);{$ENDIF}
    //if Dummy<>w then raise EFileCorrupt.Create('Illegal Id in procedure ImportWR1(var S:TStream)');
  end;

// GetInt reads an integer
  function GetOldInt:Int16; begin S.Read(Result, 2) end;

begin
  Result:=TRUE;
  Ok:=TRUE;
  try
    S:=TMemoryStream.Create;
    S.LoadFromFile(FName);
    S.Read(Header, SizeOf(Header));
    if StrComp(Header.Kennung, ImportRLYKennung) = 0 then begin
      Eat(ImportAnlageID);
      S.Read(GridInfo, SizeOf(GridInfo));
      S.Read(TextInfo, SizeOf(TextInfo));
      // Read The essential of a collection
      Eat(ImportCollID); Number:=GetOldInt; Eat(ImportDummy);Eat(ImportDummy);
      if S.Position>=S.Size then raise Exception.Create('');
      {$IFDEF DEBUG} WriteLn(Number); {$ENDIF}
      for I:=0 to Number-1 do begin
        Eat(ImportSchienenID);
        AInstance:=TInstance.Create;
        if AInstance.ImportWR1(TStream(S)) then begin
          AddElement(AInstance, FALSE);
        end else begin
          AInstance.Free;
          Ok:=FALSE;
        end;
      end;
      if not Ok then begin
        Inform(TransStr('[ModuleError]'));
        PlanDestroyed:=TRUE;
      end;
    end else begin Result:=FALSE; Exit; end; // we must go out before The text importation
  except
    Result:=FALSE;
  end;

  // importing The text
  try
    // Read The essential of a collection
    Eat(ImportCollID); Number:=GetOldInt; Eat(ImportDummy);Eat(ImportDummy);
    ClearUserTexts;
    for I:=0 to Number-1 do begin
      Eat(ImportDummy);
      S.Read(X, SizeOf(X));
      S.Read(y, SizeOf(y));
      j:=0; TempStr:='';
      repeat
        if S.Position>=S.Size then raise Exception.Create('');
        S.Read(C, SizeOf(C));
        TempStr:=TempStr+C;
        Inc(j);
      until (j>255) or (C=#0);
      delete(TempStr, Length(TempStr), 1); 
      T:=TUserTextEntry.Create;
      T.P.X:=X; T.P.y:=y;
      T.S:=TempStr;
      UserTexts.Add(T);
    end;
  except end;
  S.Free;
  IsSaved:=Result;
end;

procedure TPlan.Draw(X:TCanvas; Ax, Ay, fx, fy:Double; Overview, NoSelection:boolean; Exception:integer);
var I:integer;
    Inst:TInstance;
    DI:TDrawInfo;
    Old:boolean;
    OT:boolean;
begin
  SetDrawInfo;
  DI:=DrawInfo;
  if Overview then begin
    DI.DoubleDraw:=CDrawInner;
    DI.ShowRailNames:=FALSE;
    DI.Precision:=16;
    DI.Exact:=TRUE;
    DI.CnctDraw:=CDrawCnctSeparator;
    DI.OverRuleColor:=FALSE;
  end;
  Old:=DI.OverRuleColor;
  OT:=DI.TextColorOverrule;
  X.Font.Height:=Round(ABS(GlobalRailNrHeight*fy));
  X.Font.Name:='Arial';

  for I:=0 to InstanceElements.Count-1 do begin
    if I=Exception then Continue;
    Inst:=TInstance(InstanceElements.Items[I]);
    if Assigned(Inst) then begin
      if Overview or (not GlobalPlanUseLayers) or ((Inst.Layer>=0) and (Inst.Layer<Layers.Size) and Layers[Inst.Layer]) then begin
        Inst.Draw(X, Ax, Ay, fx, fy, Selected[I] and (not NoSelection), @DI)
      end else begin
        if GlobalPlanGrayLayer then begin
          DI.TextColorOverrule:=TRUE;
          DI.OverRuleColor:=TRUE;
          Inst.Draw(X, Ax, Ay, fx, fy, Selected[I] and (not NoSelection), @DI);
          DI.OverRuleColor:=Old;
          DI.TextColorOverrule:=OT;
        end;
      end;
    end;
  end;
end;

procedure TPlan.SpeedDraw(X:TCanvas; Ax, Ay, fx, fy:Double; Overview, NoSelection:boolean; Exception:integer; aTL, aDR:TDoublePoint);
var I:integer;
    Inst:TInstance;
    DI:TDrawInfo;
    Old:boolean;
    OT:boolean;
begin
  SetDrawInfo;
  DI:=DrawInfo;
  if Overview then begin
    DI.DoubleDraw:=CDrawInner;
    DI.ShowRailNames:=FALSE;
    DI.Precision:=16;
    DI.Exact:=TRUE;
    DI.CnctDraw:=CDrawCnctSeparator;
  end;
  Old:=DI.OverRuleColor;
  OT:=DI.TextColorOverrule;
  X.Font.Height:=Round(ABS(GlobalRailNrHeight*fy));
  X.Font.Name:='Arial';
  for I:=0 to InstanceElements.Count-1 do begin
    if I=Exception then Continue;
    Inst:=TInstance(InstanceElements.Items[I]);
    if Assigned(Inst) then begin
      if (Inst.UL.X>aDR.X) or (Inst.LR.X<aTL.X) or (Inst.UL.y>aDR.y) or (Inst.LR.y<aTL.y) then Continue;
      if Overview or (not GlobalPlanUseLayers) or ((Inst.Layer>=0) and (Inst.Layer<Layers.Size) and Layers[Inst.Layer]) then begin
        Inst.Draw(X, Ax, Ay, fx, fy, Selected[I] and (not NoSelection), @DI)
      end else begin
        if GlobalPlanGrayLayer then begin
          DI.OverRuleColor:=TRUE;
          DI.TextColorOverrule:=TRUE;
          Inst.Draw(X, Ax, Ay, fx, fy, Selected[I] and (not NoSelection), @DI);
          DI.OverRuleColor:=Old;
          DI.TextColorOverrule:=OT;
        end;
      end;
    end;
  end;
end;

procedure TPlan.DrawTable(X:TCanvas; Ax, Ay, fx, fy:Double; Selected:integer; Numbers:boolean);
var I   : integer;
    MP  : TDoublePoint;
    S   : string;
    L   : Double;
    Prev: integer;
begin
  // Who cares about a table with no area
  if TablePolygon.Count<3 then Exit;
  X.MoveTo(Round((TPointObject(TablePolygon[0]).P.X+Ax)*fx), Round((TPointObject(TablePolygon[0]).P.y+Ay)*fy));
  if 0=Selected then X.Pen.Color:=clRed else X.Pen.Color:=clBlack;
  for I:=1 to TablePolygon.Count-1 do begin
    X.LineTo(Round((TPointObject(TablePolygon[I]).P.X+Ax)*fx), Round((TPointObject(TablePolygon[I]).P.y+Ay)*fy));
    if I=Selected then X.Pen.Color:=clRed else X.Pen.Color:=clBlack;
  end;
  X.LineTo(Round((TPointObject(TablePolygon[0]).P.X+Ax)*fx), Round((TPointObject(TablePolygon[0]).P.y+Ay)*fy));

  if Numbers then begin
    Prev:=SetBKMode(X.Handle, TRANSPARENT);
    X.Font.Name:='Arial';
    X.Font.Height:=Round(ABS(GlobalUserTXTHeight*fy));
    for I:=1 to TablePolygon.Count do begin
      MP:=Middle(TPointObject(TablePolygon[I-1]).P, TPointObject(TablePolygon[I MOD TablePolygon.Count]).P);
      L:=CalcDist(TPointObject(TablePolygon[I-1]).P, TPointObject(TablePolygon[I MOD TablePolygon.Count]).P);
      Str(L:0:2, S);
      X.TextOut(Round((MP.X+Ax)*fx), Round((MP.y+Ay)*fy), S);
    end;
    SetBKMode(X.Handle, Prev);
  end;
end;

procedure TPlan.DrawUserText(X:TCanvas; Ax, Ay, fx, fy:Double);
var I   : integer;
    MP  : TDoublePoint;
    S   : string;
    Prev: integer;
begin
  Prev:=SetBKMode(X.Handle, TRANSPARENT);
  X.Font.Height:=Round(GlobalUserTXTHeight*fx);
  X.Font.Name:='Arial';
  for I:=0 to UserTexts.Count-1 do begin
    MP:=TUserTextEntry(UserTexts[I]).P;
    S:=TUserTextEntry(UserTexts[I]).S;
    try
      X.TextOut(Round((MP.X+Ax)*fx), Round((MP.y+Ay)*fy), S);
    except end;
  end;
  SetBKMode(X.Handle, Prev);
end;

procedure TPlan.UpdateSingle(Nr:integer; X:TCanvas; Ax, Ay, fx, fy:Double; Overview, Unselected:boolean);
var Inst:TInstance;
    DI:TDrawInfo;
    Old:boolean;
    OT:boolean;
begin
  DI:=DrawInfo;
  if Overview then begin
    DI.DoubleDraw:=CDrawInner;
    DI.ShowRailNames:=FALSE;
    DI.Spurweite:=1;
    DI.Precision:=16;
    DI.Exact:=TRUE;
    DI.CnctDraw:=CDrawCnctSeparator;
  end;
  X.Font.Height:=Round(ABS(GlobalRailNrHeight*fy));
  X.Font.Name:='Arial';
  try
    Inst:=TInstance(InstanceElements.Items[Nr]);
    if Assigned(Inst) then begin
      if Unselected then begin
        if (not GlobalPlanUseLayers) or ((Inst.Layer>=0) and (Inst.Layer<Layers.Size) and Layers[Inst.Layer]) then begin
          Inst.Draw(X, Ax, Ay, fx, fy, FALSE, @DI)
        end else begin
          if GlobalPlanGrayLayer then begin
            Old:=DI.OverRuleColor;
            OT:=DI.TextColorOverrule;
            DI.OverRuleColor:=TRUE;
            DI.TextColorOverrule:=TRUE;
            Inst.Draw(X, Ax, Ay, fx, fy, FALSE, @DI);
            DI.OverRuleColor:=Old;
            DI.TextColorOverrule:=OT;
          end;
        end;
      end else begin
        if (not GlobalPlanUseLayers) or ((Inst.Layer>=0) and (Inst.Layer<Layers.Size) and Layers[Inst.Layer]) then begin
          Inst.Draw(X, Ax, Ay, fx, fy, Selected[Nr], @DI)
        end else begin
          if GlobalPlanGrayLayer then begin
            Old:=DI.OverRuleColor;
            OT:=DI.TextColorOverrule;
            DI.OverRuleColor:=TRUE;
            DI.TextColorOverrule:=TRUE;
            Inst.Draw(X, Ax, Ay, fx, fy, Selected[Nr], @DI);
            DI.OverRuleColor:=Old;
            DI.TextColorOverrule:=OT;
          end;
        end;
      end;
    end;
  except
  // Toooooo bad
  end;
end;

procedure TPlan.CopySelected;
var I, j     : integer;
    IntNr    : integer;
    Inst     : TInstance;
    NewInst  : TInstance;
    DummyInst: TInstance;
    Anz      : integer;
    TempS    : TStream;
    TC       : TConnector;

begin
// Clear The internal Clipboard;
  GlobalClipboard.Free; GlobalClipboard:=TMemoryStream.Create;
// reserve some space
  GlobalClipboard.Write(Anz, SizeOf(integer));
  Anz:=0;
  for I:=0 to InstanceElements.Count-1 do begin
    if Selected[I] then begin
      Inst:=TInstance(InstanceElements.Items[I]);
      if Assigned(Inst) then begin
        Inc(Anz);

      // make a temporary copy of The instance
        TempS:=TMemoryStream.Create;
        Inst.ToStream(TempS);
        // rewind stream
        TempS.Seek(0,0);
        NewInst:=TInstance.Create;
        NewInst.FromStream(TempS);
        TempS.Free;
      // now mark The outer connectors as Unused
        for j:=0 to NewInst.ConnectorCount-1 do begin
          NewInst.GetConnector(j, TC);
          // try getting The current internal Nr of Id (TC.Nr)
          if FindElementWithID(TC.Nr, DummyInst, IntNr) then begin
            if not Selected[IntNr] then begin
              TC.Nr:=-1;
              NewInst.SetEntireConnector(j, TC);
            end;
          end;
        end;
      // Save to stream
        NewInst.ToStream(GlobalClipboard);
      // Kill temporary instance
        NewInst.Free;
      end;
    end;
  end;
  GlobalClipboard.Seek(0,0);
// Write The effective Nr of Elements to stream
  GlobalClipboard.Write(Anz, SizeOf(integer));
// and back again
  GlobalClipboard.Seek(0,0);
end;

function TPlan.GetSelectedFreeCnctList(var Res:TList):boolean;
var I        : integer;
    TempList : TList;
begin
  try
    TempList:=TList.Create;
    for I:=0 to InstanceElements.Count-1 do if Selected[I] then TempList.Add(InstanceElements.Items[I]);
    CreateCnctList(TempList, Res);
    TempList.Free;
  except end;
  Result:=Res.Count>0;
end;

function TPlan.GetAllFreeCnctList(var Res:TList):boolean;
var I        : integer;
    TempList : TList;
begin
  try
    TempList:=TList.Create;
    for I:=0 to InstanceElements.Count-1 do TempList.Add(InstanceElements.Items[I]);
    CreateCnctList(TempList, Res);
    TempList.Free;
  except end;
  Result:=Res.Count>0;
end;


// Produce a List of Instances for further use
procedure TPlan.GetInstanceList(var X:TList);
var Anz, I : integer;
    Inst   : TInstance;
begin
  X:=TList.Create;
  if GlobalClipboard.Size<=0 then Exit;
  GlobalClipboard.Read(Anz, SizeOf(integer));
  for I:=0 to Anz-1 do begin
    Inst:=TInstance.Create;
    Inst.FromStream(GlobalClipboard);
    X.Add(Inst);
  end;
  GlobalClipboard.Seek(0,0);
end;

// Take a List of Instances and Create a List of {Point, Angle, Id, Nr, TempAngle} in a
// clockwise sorted order
  function clockwise(Item1, Item2: Pointer): integer;
  begin
    if Assigned(Item1) and Assigned(Item2) then begin
      Result:=0;
      if TConnectorListPoint(Item1).TempAngle<TConnectorListPoint(Item2).TempAngle then Result:=-1;
      if TConnectorListPoint(Item1).TempAngle>TConnectorListPoint(Item2).TempAngle then Result:=1;
    end else Result:=0;
  end;

procedure TPlan.CreateCnctList(X: TList; var Res:TList);
//type TListSortCompare = function (Item1, Item2: Pointer): integer;
var CnctObj : TConnectorListPoint;
    Inst    : TInstance;
    I, j    : integer;
    TC      : TConnector;
    MP      : TDoublePoint;
begin
  Res:=TList.Create;
  MP.X:=0;MP.y:=0;
  for I:=0 to X.Count-1 do begin
//    WriteLn('I=',I);
    Inst:=TInstance(X.Items[I]);
    if Assigned(Inst) then begin
      for j:=0 to Inst.ConnectorCount-1 do begin
        Inst.GetConnector(j, TC);
        if TC.Nr = -1 then begin
//          WriteLn('Ein Mglicher Anschluss');
          CnctObj:=TConnectorListPoint.Create;
          CnctObj.Cnct:=j;
          CnctObj.Id:=Inst.Id;
          Inst.GetConnectorPos(j, CnctObj.P, CnctObj.a);
          // Center of mass
          MP.X:=MP.X+CnctObj.P.X;
          MP.y:=MP.y+CnctObj.P.y;
          Res.Add(CnctObj);
        end;
      end;
    end;
  end;
  if Res.Count > 0 then begin
    MP.X:=MP.X/Res.Count;
    MP.y:=MP.y/Res.Count;
    for I:=0 to Res.Count-1 do begin
      CnctObj:=TConnectorListPoint(Res.Items[I]);
      CnctObj.TempAngle:=ArcTan2(CnctObj.P.X-MP.X, CnctObj.P.y-MP.y);
    end;
    Res.Sort(clockwise);
  end;
end;

procedure TPlan.SetInstancesPosTo(List, Folge:TList; p1:TDoublePoint; a1:Double; p2:TDoublePoint; a2:Double);
var I          : integer;
    Inst       : TInstance;
    DeltaAngle : Double;
    dx, dy     : Double;
    TF         : TConnectorListPoint;
begin
  DeltaAngle:=FixedAngle(a1-a2);
  dx:=p2.X-p1.X;
  dy:=p2.y-p1.y;
  for I:=0 to List.Count-1 do begin
    Inst:=TInstance(List.Items[I]);
    if Assigned(Inst) then with Inst do begin
      Angle:=FixedAngle(Angle-DeltaAngle);
      Position:=GlobalRotatePoint(Position, p1, -DeltaAngle);
      Position.X:=Position.X+dx;
      Position.y:=Position.y+dy;
      TextPos:=GlobalRotatePoint(TextPos, p1, -DeltaAngle);
      TextPos.X:=TextPos.X+dx;
      TextPos.y:=TextPos.y+dy;
      if Inst.IsFlex then Inst.FlexAddition.Turnmove(dx, dy, DeltaAngle, p1);
      CheckIn;
    end;
  end;
  for I:=0 to Folge.Count-1 do begin
    TF:=TConnectorListPoint(Folge.Items[I]);
    if Assigned(TF) then with TF do begin
      a:=FixedAngle(a-DeltaAngle);
      P:=GlobalRotatePoint(P, p1, -DeltaAngle);
      P.X:=P.X+dx;
      P.y:=P.y+dy;
    end;
  end;
end;

function TPlan.IsConnectible(I1: TInstance; CNr1: integer; I2: TInstance; CNr2: integer; DistTol, AngleTol:Double):boolean;
var C1, C2: TConnector;
    p1, p2: TDoublePoint;
    a1, a2: Double;
    TransRail:boolean;
begin
  Result:=FALSE;
  if not I1.GetConnector(CNr1, C1) then Exit;
  if not I2.GetConnector(CNr2, C2) then Exit;
  if (C1.Nr<>-1) or (C2.Nr<>-1) then Exit;
  if not I1.GetConnectorPos(CNr1, p1, a1) then Exit;
  if not I2.GetConnectorPos(CNr2, p2, a2) then Exit;
  try
		// do not fortget turning around one of the connectors
    if FixedAngle(FixedAngle(a1)-FixedAngle(a2+pi))>FixedAngle(AngleTol) then Exit;
    if (ABS(p2.X-p1.X)>DistTol) or (ABS(p2.y-p1.y)>DistTol) then Exit;
    if ABS(p2.y-p1.y)>0 then begin
      if Hypot(ABS(p2.X-p1.X), ABS(p2.y-p1.y))>DistTol then Exit;
    end else if ABS(p2.X-p1.X)>DistTol then Exit;
		// Result is only TRUE if none of the previous tests fails
    TransRail:=FALSE;
    if I1.CacheElement.Info.TheType=UebergangsStueck then
      if (StrPas(I1.CacheElement.Info.Uebergang)=StrPas(I2.ModuleName)) then TransRail:=TRUE;
    if I2.CacheElement.Info.TheType=UebergangsStueck then
      if (StrPas(I2.CacheElement.Info.Uebergang)=StrPas(I1.ModuleName)) then TransRail:=TRUE;
    if (not TransRail) and (GlobalCheckDiffMod) then begin
      if StrPas(I1.ModuleName)<>StrPas(I2.ModuleName) then
        if Assigned(OkNotTheSameModules) then
          if not OkNotTheSameModules(StrPas(I1.ModuleName), StrPas(I2.ModuleName), p1) then Exit;

      if I1.ModuleCache.Info.System<>I2.ModuleCache.Info.System then begin
        if Assigned(OkNotTheSameSystems) then
          if not OkNotTheSameSystems(StrPas(I1.ModuleName), StrPas(I2.ModuleName), p1) then Exit;
      end;
    end;
    Result:=TRUE;
  except
		// if this fails the distance was much to great and the result "FALSE" is correct
    // just leave
  end;
end;

procedure TPlan.AddInstance(Inst:TInstance; Selectem:boolean; Undoable, NewID:boolean; ResultList:TList);
var I, j, k: integer;
    Comp   : TInstance;
    X      : TPrepDelInstance;
    P      : TDoublePoint;
    a      : Double;
begin
  if not Assigned(Inst) then Exit;
  if NewID then Inst.Id:=GetUniqueID;
  // The new element is nowhere connected
  for k:=0 to Inst.ConnectorCount-1 do Inst.SetConnector(k, -1);
  for I:=0 to InstanceElements.Count - 1 do begin
    Comp:=TInstance(InstanceElements.Items[I]);
    if not Assigned(Comp) then Continue; // skip if not Assigned
		// Take all the connectors of the Comp.-element.
    for j:=0 to Comp.ConnectorCount-1 do begin
      for k:=0 to Inst.ConnectorCount-1 do begin
        if IsConnectible(Inst, k, Comp, j, GlobalDistTol, GlobalAngleTol) then begin
          Inst.SetConnector(k, Comp.Id);
          Comp.SetConnector(j, Inst.Id);
          if Assigned(ResultList) then if Comp.GetConnectorPos(j, P, a) then ResultList.Add(TPointObject.Create(P.X, P.y));
        end;
      end;
    end;
  end;
  if Undoable then begin
    // tell Undo, what to do
    X:=TPrepDelInstance.Create; X.DeathCandidate:=Inst.Id;  UndoHelper.AddState(X);
  end;
  AddElement(Inst, Selectem);
end;

procedure TPlan.AddInstancesList(var X:TList; Selectem:boolean; ResultList:TList);
var I          : integer;
    TempCursor : TCursor;
begin
  if not Assigned(X) then Exit; // Assure X is vaild
  TempCursor:=Screen.Cursor;
  Screen.Cursor:=crHourglass;
  GlobalProgressBar.Visible:=TRUE;
  UndoHelper.Open;
  for I:=0 to X.Count-1 do begin
    if I MOD 100 = 0 then GlobalProgressBar.Position:=Round(100*I/X.Count);
    AddInstance(TInstance(X.Items[I]), Selectem, TRUE, TRUE, ResultList);
  end;
  GlobalProgressBar.Position:=100;
  X.Free;
  UndoHelper.Close;
  X:=NIL;
  GlobalProgressBar.Visible:=FALSE;
  Screen.Cursor:=TempCursor;
end;

procedure TPlan.SelectAll;
var I:integer;
    Inst:TInstance;
begin
  for I:=0 to Selected.Size-1 do begin
    Inst:=TInstance(InstanceElements.Items[I]);
    if Assigned(Inst) then begin
      Selected[I]:=(not GlobalPlanUseLayers) or ((Inst.Layer>=0) and (Inst.Layer<Layers.Size) and Layers[Inst.Layer]);
    end else Selected[I]:=FALSE;
  end;
end;

procedure TPlan.ClearSelection;
var I:integer;
begin
  for I:=0 to Selected.Size-1 do Selected[I]:=FALSE;
end;

function TPlan.DeleteSelected:integer;
var I, j  : integer;
    Cnt   : integer;
    Nr    : integer;
    SelNr : integer;
    C     : TConnector;
    Inst  : TInstance;
begin
  // delete down to !!
  Cnt:=0;
  Nr:=-1;
  UndoHelper.Open;
  for I:=Selected.Size-1 downto 0 do if Selected[I] then begin
    if Cnt=0 then begin
      if GetElement(I, Inst) then begin
        for j:=0 to Inst.ConnectorCount-1 do begin
          if Inst.GetConnector(j, C) then begin
            if C.Nr<>-1 then begin
              Nr:=C.Nr;
              Break;
            end;
          end;
        end;
      end;
    end;
    DeleteElement(I, TRUE, TRUE);
    Inc(Cnt);
  end;
  UndoHelper.Close;
  ClearSelection;
   Result:=-1;
  if Nr>-1 then if FindElementWithID(Nr, Inst, SelNr) then begin
    Selected[SelNr]:=TRUE;
    Result:=SelNr;
  end;
end;

function TPlan.OneSelectedTyp(var Typ:TImportSchienenTyp; var Flex:boolean):boolean;
var I, C : integer;
    Nr   : integer;
    Tinst: TInstance;
begin
  Result:=FALSE;C:=0;Nr:=0;
  for I:=0 to Selected.Size-1 do
    if Selected[I] then begin
      Inc(C);
      if C=1 then begin
        Result:=TRUE;
        Nr:=I;
      end else begin
        Result:=FALSE;
        Exit;
      end;
  end;
  if GetElement(Nr, Tinst) then begin
    Typ:=Tinst.CacheElement.Info.TheType;
    Flex:=Tinst.IsFlex;
  end else Result:=FALSE;
end;

procedure TPlan.Invert(Nr:integer);
begin
  try
    Selected[Nr]:=not Selected[Nr];
  except
		// if anything goes wrong Nr was out of range, who cares
	end;
end;

procedure TPlan.Select(Nr:integer);
begin
	try
		Selected[Nr]:=TRUE;
	except
		// if anything goes wrong Nr was out of range, who cares
	end;
end;

procedure TPlan.Unselect(Nr:integer);
begin
	try
		Selected[Nr]:=FALSE;
	except
		// if anything goes wrong Nr was out of range, who cares
	end;
end;

procedure TPlan.CalcRectAroundInstList(CurrentInstList:TList; var TUL, TLR:TDoublePoint);
var I     : integer;
		Inst  : TInstance;
		a, B  : TSinglePoint;
		First : boolean;
begin
	First:=TRUE;
	TUL.X:=0; TUL.y:=0; // Prevent div by zeros if something fails
	TLR.X:=1; TLR.y:=1;
	if not Assigned(CurrentInstList) then Exit;
	for I:=0 to CurrentInstList.Count-1 do begin
		Inst:=TInstance(CurrentInstList.Items[I]);
		if Assigned(Inst) then begin
			a:=Inst.UL; B:=Inst.LR;
			if First then begin TUL.X:=a.X;TUL.y:=a.y; TLR.X:=B.X;TLR.y:=B.y; First:=FALSE; end;
			if TUL.X>a.X then TUL.X:=a.X;
			if TUL.y>a.y then TUL.y:=a.y;
			if TLR.X<B.X then TLR.X:=B.X;
			if TLR.y<B.y then TLR.y:=B.y;
		end;
	end;
end;

procedure TPlan.CalcRectAroundSelection;
var I     : integer;
		Inst  : TInstance;
		a, B  : TSinglePoint;
    First : boolean;
begin
  First:=TRUE;
  SUL.X:=0; SUL.y:=0; // Prevent div by zeros if something fails
  SLR.X:=1; SLR.y:=1;
  for I:=0 to InstanceElements.Count-1 do if Selected[I] then begin
    Inst:=TInstance(InstanceElements.Items[I]);
    if Assigned(Inst) then begin
      a:=Inst.UL; B:=Inst.LR;
      if First then begin SUL.X:=a.X;SUL.y:=a.y; SLR.X:=B.X;SLR.y:=B.y; First:=FALSE; end;
      if SUL.X>a.X then SUL.X:=a.X;
      if SUL.y>a.y then SUL.y:=a.y;
      if SLR.X<B.X then SLR.X:=B.X;
      if SLR.y<B.y then SLR.y:=B.y;
    end;
  end;
end;


function TPlan.GetElement(Nr:integer; var I:TInstance):boolean;
begin
  Result:=FALSE;
  try
    I:=TInstance(InstanceElements.Items[Nr]);
    if Assigned(I) then Result:=TRUE;
  except end; // if Nr was invaild;
end;

function TPlan.FindElementWithID(Id:integer; var Inst:TInstance; var Nr:integer):boolean;
var I:integer;
begin
  Result:=FALSE;
  for I:=0 to InstanceElements.Count-1 do begin
    Inst:=TInstance(InstanceElements.Items[I]);
    if Assigned(Inst) and (Inst.Id=Id) then begin
      Result:=TRUE;
      Nr:=I;
      Exit;
    end;
  end;
end;

function TPlan.IsHit(Nr:integer;ClX, ClY:longint; Ax, Ay, fx, fy:Double):boolean;
var Inst:TInstance;
begin
	try
		Inst:=TInstance(InstanceElements.Items[Nr]);
		if (not GlobalPlanUseLayers) or ((Inst.Layer>=0) and (Inst.Layer<Layers.Size) and Layers[Inst.Layer]) then
			Result:=Inst.IsHit(ClX, ClY, Ax, Ay, fx, fy, @DrawInfo)
		else Result:=FALSE;
	except
		Result:=FALSE; // make sure no error can Result in TRUE
	end;
end;

function TPlan.InRect(Nr:integer; UL, DR:TDoublePoint):boolean;
var Inst:TInstance;
begin
	try
		Inst:=TInstance(InstanceElements.Items[Nr]);
		if (not GlobalPlanUseLayers) or ((Inst.Layer>=0) and (Inst.Layer<Layers.Size) and Layers[Inst.Layer]) then
			Result:=Inst.IsInRect(UL, DR)
		else Result:=FALSE;
	except
		Result:=FALSE; // make sure no error can Result in TRUE
	end;
end;

function TPlan.Hit(ClX, ClY:longint; Ax, Ay, fx, fy:Double; var Nr:integer):boolean;
var I:integer;
		Inst:TInstance;
begin
	Result:=FALSE;
	for I:=0 to InstanceElements.Count-1 do begin
		ActualSearch:=(ActualSearch+1) MOD InstanceElements.Count;
		Inst:=TInstance(InstanceElements.Items[ActualSearch]);
		if Assigned(Inst) then begin
			if (not GlobalPlanUseLayers) or ((Inst.Layer>=0) and (Inst.Layer<Layers.Size) and Layers[Inst.Layer]) then
			 if Inst.IsHit(ClX, ClY, Ax, Ay, fx, fy, @DrawInfo) then begin
				 Result:=TRUE;
				 Nr:=ActualSearch;
				 Exit;
			 end;
		end;
	end;
end;

procedure TPlan.SelectedToMeta;
var I          : integer;
		Meta       : TMetafile;
		MetaCanvas : TCanvas;
		LocalXSp,
		LocalYSp   : Double;
    ZF         : Double;
    Breite,
    Hoehe      : integer;

  function ShowAllFactor:Double;
  var xs,ys:Double;
  begin
    xs:=1; ys:=1;
    if (ABS(SLR.X-SUL.X)>0) then xs:=(Breite-20)/ABS(SLR.X-SUL.X);
    if (ABS(SLR.y-SUL.y)>0) then ys:=(Hoehe-20)/ABS(SLR.y-SUL.y);
    if xs<ys then Result:=xs else Result:=ys;
  end;

begin
    try
      Meta:=TMetafile.Create;
      CalcRectAroundSelection;
      Breite:=10000; Hoehe:=Round(Breite/(SLR.X-SUL.X)*(SLR.y-SUL.y)*1.1);
      Meta.Height:=Hoehe;
      Meta.Width:=Breite;

      MetaCanvas:=TMetaFileCanvas.Create(Meta, 0);

      ZF:=ShowAllFactor;
      LocalXSp:=10/ZF-SUL.X;
      LocalYSp:=10/ZF-SUL.y;
      for I:= InstanceElements.Count-1 downto 0 do begin
        if Selected[I] then begin
           UpdateSingle(I, MetaCanvas, LocalXSp, LocalYSp, ZF, ZF, FALSE, TRUE);
        end;
      end;
      MetaCanvas.Free;
      Clipboard.Assign(Meta);
      Meta.Free;
    except
      // Picture is not in Clipboard, Who cares...
    end;
end;

procedure TPlan.AllToMeta;
var I          : integer;
    Meta       : TMetafile;
    MetaCanvas : TCanvas;
    LocalXSp,
    LocalYSp   : Double;
    ZF         : Double;
    Breite,
    Hoehe      : integer;

  function ShowAllFactor:Double;
  var xs,ys:Double;
  begin
    xs:=1; ys:=1;
    if (ABS(SLR.X-SUL.X)>0) then xs:=(Breite-20)/ABS(SLR.X-SUL.X);
    if (ABS(SLR.y-SUL.y)>0) then ys:=(Hoehe-20)/ABS(SLR.y-SUL.y);
    if xs<ys then Result:=xs else Result:=ys;
  end;

begin
    try
      Meta:=TMetafile.Create;
      CalcRectAround;
      Breite:=10000; Hoehe:=Round(Breite/(LR.X-UL.X)*(LR.y-UL.y)*1.1);
      Meta.Height:=Hoehe;
      Meta.Width:=Breite;

      MetaCanvas:=TMetaFileCanvas.Create(Meta, 0);

      ZF:=ShowAllFactor;
      LocalXSp:=10/ZF-UL.X;
      LocalYSp:=10/ZF-UL.y;
      for I:= InstanceElements.Count-1 downto 0 do begin
        UpdateSingle(I, MetaCanvas, LocalXSp, LocalYSp, ZF, ZF, FALSE, TRUE);
      end;
      DrawTable(MetaCanvas, LocalXSp, LocalYSp, ZF, ZF, -1, GlobalTableSizeNrs);
      if GlobalPlanUserTXT then DrawUserText(MetaCanvas, LocalXSp, LocalYSp, ZF, ZF);
      MetaCanvas.Free;
      Clipboard.Assign(Meta);
      Meta.Free;
		except
      // Picture is not in Clipboard, Who cares...
    end;
end;

procedure TPlan.ClearPlan;
var I:longint;
begin
  IsSaved:=FALSE;
  for I:=InstanceElements.Count-1 downto 0 do begin
    TInstance(InstanceElements.Items[I]).Free;
    InstanceElements.delete(I);
  end;
end;

procedure TPlan.BuildStandardTable;
begin
  IsSaved:=FALSE;
  // Clear current table
  ClearTablePolygon;
  CalcRectAround;
  if InstanceElements.Count=0 then begin
    TablePolygon.Add(TPointObject.Create(0, 0));
    TablePolygon.Add(TPointObject.Create(1000, 0));
    TablePolygon.Add(TPointObject.Create(1000, 1000));
    TablePolygon.Add(TPointObject.Create(0, 1000));
  end else begin
    TablePolygon.Add(TPointObject.Create(UL.X, UL.y));
    TablePolygon.Add(TPointObject.Create(UL.X, LR.y));
    TablePolygon.Add(TPointObject.Create(LR.X, LR.y));
    TablePolygon.Add(TPointObject.Create(LR.X, UL.y));
  end;
end;

procedure TPlan.ClearTablePolygon;
var I:longint;
begin
  IsSaved:=FALSE;
  for I:=TablePolygon.Count-1 downto 0 do begin
    TPointObject(TablePolygon[I]).Free;
		TablePolygon.delete(I);
	end;
end;

function TPlan.PointToGridPoint(X:TDoublePoint):TDoublePoint;
begin
	if GlobalPlanGridSize>0 then begin
		Result.X:=Round(X.X/GlobalPlanGridSize)*GlobalPlanGridSize;
		Result.y:=Round(X.y/GlobalPlanGridSize)*GlobalPlanGridSize;
	end else Result:=X;
end;

procedure TPlan.CalcInventory(All, Unused:boolean; List:TStringList);
var I, j  :integer;
		Tinst : TInstance;
		M     : TModule;
		Cl    : TStringList;
begin
	Cl:=TStringList.Create; Cl.Duplicates:=dupIgnore; Cl.sorted:=TRUE;

	Inventory.ResetCount;

	if All then begin
		// Search at least all the modules that are used
		for I:=0 to InstanceElements.Count-1 do
			if GetElement(I, Tinst) and Assigned(Tinst) then
				Cl.Add(StrPas(Tinst.ModuleName));

		// include the possibly different list of selected modules
		if Unused and Assigned(List) then Cl.AddStrings(List);

		// make all the elements known to the inventory
		for I:=0 to Cl.Count-1 do begin
			if GlobalModules.GetModule(Cl[I], M) then begin
				for j:=0 to M.Elements.Count-1 do Inventory.Know(M.Info.Name, M.Elements[j]);
			end;
		end;
	end;

	for I:=0 to InstanceElements.Count-1 do begin
		if GetElement(I, Tinst) then
			Inventory.Enter(StrPas(Tinst.ModuleName), StrPas(Tinst.ElementName));
			Inventory.IncLength(StrPas(Tinst.ModuleName), StrPas(Tinst.ElementName), Tinst.GetLength);
	end;
	Cl.Free;
	Inventory.Sort;
end;

procedure TPlan.Undo;
begin
	UndoHelper.Undo;
end;

procedure TPlan.UndoSomething(X:TObject);
var Nr:integer;
		Tinst:TInstance;
begin
	if X is TPrepDelInstance then begin
		if FindElementWithID(TPrepDelInstance(X).DeathCandidate, Tinst, Nr) then
			DeleteElement(Nr, FALSE, TRUE);
		TPrepDelInstance(X).Free;
		Exit;
	end;

	if X is TPrepAddInstance then begin
		Tinst:=TInstance.Create;
		TPrepAddInstance(X).Backup.Seek(0,0);
		Tinst.FromStream(TPrepAddInstance(X).Backup);
		AddInstance(Tinst, TRUE, FALSE, FALSE, NIL);
		TPrepAddInstance(X).Free;
		Exit;
	end;
	if X is TPrepColorUndo then begin
		if FindElementWithID(TPrepColorUndo(X).Candidate, Tinst, Nr) then
			Tinst.Color:=TPrepColorUndo(X).Color;
		TPrepColorUndo(X).Free;
		Exit;
	end;
end;

procedure TPlan.TextsToStream(S:TStream);
var I, L:integer;
		T:TUserTextEntry;
begin
	L:=UserTexts.Count;
	S.Write(L, SizeOf(L));
	for I:=0 to L-1 do begin
		T:=TUserTextEntry(UserTexts[I]);
		S.Write(T.P, SizeOf(T.P));
		StringToStream(T.S, S);
	end;
end;

procedure TPlan.ClearUserTexts;
var I:integer;
begin
	for I:=UserTexts.Count-1 downto 0 do begin
		TUserTextEntry(UserTexts[I]).Free;
		UserTexts.delete(I);
	end;
	UserTexts.Pack;
end;

procedure TPlan.TextsFromStream(S:TStream);
var I, L:integer;
		T:TUserTextEntry;
begin
	S.Read(L, SizeOf(L));
	for I:=0 to L-1 do begin
		if S.Position>=S.Size then raise Exception.Create('FileError');
		T:=TUserTextEntry.Create;
		S.Read(T.P, SizeOf(T.P));
		T.S:=StringFromStream(S);
		if S.Position<=S.Size then UserTexts.Add(T);
	end;
end;

procedure TPlan.ShowAllLayers;
var I:integer;
begin
	 for I:=0 to Layers.Size-1 do Layers[I]:=TRUE;
end;

function TPlan.CalcLength:Double;
var I:integer;
begin
	Result:=0;
	for I:=0 to InstanceElements.Count-1 do if Assigned(InstanceElements[I]) then
		Result:=Result+TInstance(InstanceElements[I]).GetLength;
end;

function TPlan.CalcRealLength:Double;
var I:integer;
begin
	Result:=0;
	for I:=0 to InstanceElements.Count-1 do if Assigned(InstanceElements[I]) then begin
		Result:=Result+TInstance(InstanceElements[I]).GetLength*TInstance(InstanceElements[I]).ModuleCache.Factor;
	end;
end;

destructor TPlan.Free;
begin
	Inventory.Free;
	ClearPlan;
	InstanceElements.Free;
	Selected.Free;
	ClearTablePolygon;
	TablePolygon.Free;
	UndoHelper.Free;
	Layers.Free;
	ClearUserTexts;
	UserTexts.Free;
end;

end.
