(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit PrintPreview;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ExtCtrls, printers, RailyTypes, StdCtrls, Spin, Globals,
  ComCtrls;


type TAdditionalPrintInfo = record name, author:string; end;

type
	TPreviewForm = class(TForm)
		Panel1: TPanel;
		PPVPrintMulti: TSpeedButton;
		PPVSelect: TSpeedButton;
		PrinterSetupDialog1: TPrinterSetupDialog;
		PPVAll: TSpeedButton;
		PPVLabel1: TLabel;
    PREVFactorEdit: TEdit;
    PREVPageNr: TSpinEdit;
    PgNrLBL: TLabel;
    PREVPage: TLabel;
    PREVPGNrs: TSpeedButton;
    PPVOptions: TSpeedButton;
    PPVPrint: TSpeedButton;
    ProgressBar1: TProgressBar;
    AbortBTN: TSpeedButton;
    StopPreview: TSpeedButton;
    TL: TSpeedButton;
    TR: TSpeedButton;
    BL: TSpeedButton;
    BR: TSpeedButton;
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PPVSelectClick(Sender: TObject);
    procedure PPVAllClick(Sender: TObject);
    procedure PREVFactorEditExit(Sender: TObject);
    procedure PREVPageNrChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure PREVPGNrsClick(Sender: TObject);
    procedure PREVFactorEditChange(Sender: TObject);
    procedure PREVFactorEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PREVFactorEditKeyPress(Sender: TObject; var Key: Char);
    procedure FormActivate(Sender: TObject);
    procedure PPVPrintClick(Sender: TObject);
    procedure AbortBTNClick(Sender: TObject);
    procedure PPVOptionsClick(Sender: TObject);
    procedure StopPreviewClick(Sender: TObject);
    procedure BRClick(Sender: TObject);
  private
    { private-Deklarationen }
  public
    blatt          : TCanvas;
    dc             : HDC;
		druckbreite,
    druckhoehe     : integer;
    PixWidth,
    PixHeight      : integer;
    panelx         : TWinControl;
    CurrentPage    : integer;
    UL, DR         : TDoublePoint;
    xs, ys, xf, yf : DOUBLE;
    UserXS, UserYS : DOUBLE;
    XPages, YPages : integer;
    mmTop, mmLeft  : integer;
    Multipages     : boolean;
    Title          : string;
    InfoArrange    : integer;
    AusgabeProc    : procedure (X:TCanvas; xs, ys, xf, yf:DOUBLE; BW:boolean) of object;
    GetBoundsProc  : procedure (var UL, DR: TDoublePoint) of object;
    OutPagePrinter : procedure (X:TCanvas; DW, DH, PG:integer) of object;
    GetPageCount   : procedure (DW, DH:integer; var PC:integer) of object;
    PagePrinter    : boolean;
    Info           : TAdditionalPrintInfo;
    FirstActive    : boolean;

    { public-Deklarationen }
    procedure blattanpassen;
    function ShowAllFactor:DOUBLE;
    procedure PrepShowAll;
    procedure UpdateScaleField;
    procedure ausgabe(RealPrinter:boolean; Page:integer);
    procedure CheckRange;
    procedure DisableOptions(X:boolean);
    procedure Center;
    destructor Free;
  end;


implementation

uses OptionsDialog;

{$R *.DFM}

procedure TPreviewForm.blattanpassen;
var prnxy, formxy         : single;
		dx,dy,X,y             : integer;

begin
		druckbreite := GetDeviceCaps(printer.handle, horzsize);
		druckhoehe  := GetDeviceCaps(printer.handle, vertsize);

		PixWidth:=druckbreite*10;
		PixHeight:=druckhoehe*10;

		y := 35;
		X := 10;
		formxy := (clientWidth - X) / (clientheight - y);
		prnxy := druckbreite / druckhoehe;
		if formxy < prnxy then begin
				dx := clientWidth - 20;
				dy := round(dx / prnxy);
				y  := round((clientheight / 2 + 8) - (dy / 2));
		end else begin
				dy := clientheight - y-10;
				dx := round(dy * prnxy);
				X  := round((clientWidth / 2) - (dx / 2));
		end;
		panelx.setbounds(X,y,dx,dy);
		SetMapMode(blatt.handle,mm_anisotropic);
		SetViewportExtEX(blatt.handle,dx,dy,nil);
		SetWindowExtEX(blatt.handle,druckbreite*10,-druckhoehe*10,nil);
		Setbkmode(blatt.handle,TRANSPARENT);
		blatt.Brush.Style := bssolid;
		blatt.Brush.Color := clwhite;
		blatt.Pen.Color := clblack;
		blatt.Rectangle(0,0,druckbreite*10,-druckhoehe*10);
end;

procedure TPreviewForm.ausgabe(RealPrinter:boolean; Page:integer);
var X    : TCanvas;
		Prev : integer;
		S    : string;
		TP   : TRect;
    TI   : integer;
const W= 1000;
			H= 250;
begin
  Page:=Page-1; // User wants from 1 to n but I need from 0 to n-1
  if RealPrinter then X:=printer.Canvas else X:=blatt;

  if not PagePrinter and Assigned(AusgabeProc) then begin
    if Multipages then begin
      xs:=-(-UserXS + (Page MOD XPages)* PixWidth/xf);
      ys:=(UserYS + (Page DIV XPages)* PixHeight/yf);
    end else begin
      xs:=UserXS;
      ys:=UserYS;
    end;
    AusgabeProc(X, xs, ys, xf, yf, GlobalPrintBW);
  end;

  if PagePrinter and Assigned(OutPagePrinter) then begin
    OutPagePrinter(X, PixWidth, PixHeight, Page);
  end;

//  OH:=X.Font.Height;

  if PREVPGNrs.Down then begin
    Prev:=Setbkmode(X.handle, TRANSPARENT);
    X.Font.Name:='Arial';
    X.Font.Height:=GlobalPrintPNRSize;
    X.Font.Color:=clblack;
    S:=IntToStr(Page+1);
    X.TextOut((PixWidth - X.TextWidth(S))DIV 2, -PixHeight+X.TextHeight(S), S);
    Setbkmode(X.handle, Prev);
  end;

  if (not PagePrinter) and (InfoArrange>0) and ((not Multipages) or
     ((InfoArrange=1) and (CurrentPage=1)) or // First Page top left
     ((InfoArrange=2) and (CurrentPage=XPages))or  // First row right
     ((InfoArrange=3) and (CurrentPage=XPages*YPages-XPages+1))or  // Last row left
     ((InfoArrange=4) and (CurrentPage=XPages*YPages))) // Last Page bottom right
		 then begin
    Prev:=Setbkmode(X.handle, TRANSPARENT);
    X.Pen.Color:=clblack;
		X.Brush.Color:=clwhite;
    case InfoArrange of
      1: begin
           TP.left:=1;
           TP.top:=-(1);
           TP.right:=W;
           TP.bottom:=-(H);
         end;
      2: begin
           TP.left:=PixWidth-W;
           TP.top:=-(1);
           TP.right:=PixWidth-1;
           TP.bottom:=-(H);
         end;
      3: begin
           TP.left:=1;
           TP.top:=-(PixHeight-H);
           TP.right:=W;
           TP.bottom:=-(PixHeight-1);
         end;
      4: begin
           TP.left:=PixWidth-W;
           TP.top:=-(PixHeight-H);
           TP.right:=PixWidth-1;
           TP.bottom:=-(PixHeight-1);
         end;
    end;
    X.Rectangle(TP.left, TP.top, TP.right, TP.bottom);

    X.Font.Name:='Arial';

    S:=Info.Name;
    X.Font.Style:=X.Font.Style+[fsBold];
    X.Font.Height:=60;
    TextOut(X.handle, TP.left+10, TP.top, PCHAR(S), Length(S));

    TI:=60;

    X.Font.Style:=X.Font.Style-[fsBold];


		S:=Info.Author;
		FFont(X, 'Arial', 60, FW_Normal);
		X.TextOut(TP.left+10, TP.top-TI, S);

		TI:=TI+60;

		S:='1 : '+TRIM(PREVFactorEdit.Text);
		// Kill following zeros for aesthetic reasons
		while (Length(S)>0) and (S[Length(S)]='0') do Delete(S, Length(S), 1);
		// Kill following the point (if it is the laste Char) for aesthetic reasons
    if S[Length(S)]='.' then Delete(S, Length(S), 1);



    FFont(X, 'Arial', 60, FW_BOLD);
    X.TextOut(TP.left+10, TP.top-TI, S);
    TI:=TI+60+10;
    S:=TransStr('[Printed with RAILY]');

    FFont(X, 'Arial', 30, FW_Normal);
    X.TextOut(TP.left+10, TP.top-TI, S);

    Setbkmode(X.handle, Prev);
  end;
end;


procedure TPreviewForm.FormPaint(Sender: TObject);
begin
  blattanpassen;
  ausgabe(FALSE, CurrentPage);
end;

procedure TPreviewForm.FormCreate(Sender: TObject);
begin
  FirstActive:=TRUE;
	panelx := TWinControl.create(self);
  panelx.parent := self;
  blatt := TCanvas.create;
	blatt.handle := GetDC(panelx.handle);
  SetMapMode(blatt.handle,mm_anisotropic);
  blattanpassen;
  Multipages:=FALSE;
  CurrentPage:=1;
  PagePrinter:=FALSE;
  Title:='Hello printer';
end;

procedure TPreviewForm.PPVSelectClick(Sender: TObject);
begin
  PrinterSetupDialog1.execute;
  CurrentPage:=1;
  blattanpassen;
  CheckRange;
  Center;
  Invalidate;
end;

function TPreviewForm.ShowAllFactor:DOUBLE;
var xs,ys:DOUBLE;
begin
  xs:=1; ys:=1; Result:=1;
  if PagePrinter then EXIT;
  if not Assigned(GetBoundsProc) then EXIT;
  GetBoundsProc(UL, DR);
  if (ABS(DR.X-UL.X)>0) then xs:=(PixWidth)/ABS(DR.X-UL.X);
  if (ABS(DR.y-UL.y)>0) then ys:=(PixHeight)/ABS(DR.y-UL.y);
  if xs<ys then Result:=xs else Result:=ys;
end;

procedure TPreviewForm.PrepShowAll;
var Zoomfactor:DOUBLE;
begin
  if PagePrinter then EXIT;
  Zoomfactor:=ShowAllFactor*0.95;
  xf:=Zoomfactor;
  yf:=-xf;
  CheckRange;
  UpdateScaleField;
  Center;
end;

procedure TPreviewForm.Center;
var PW, PH:integer;
begin
  PW:=XPages*PixWidth;
  PH:=YPages*PixHeight;
  UserXS:=-UL.X+(PW/xf-(DR.X-UL.X))/2;
  UserYS:=-UL.y+(PH/ABS(yf)-(DR.y-UL.y))/2;
	Invalidate;
end;

procedure TPreviewForm.CheckRange;
begin
  if PagePrinter then begin
    if Assigned(GetPageCount) then begin
      GetPageCount(PixWidth, PixHeight, YPages);
      Multipages:=YPages>1;
      XPages:=1;
    end;
  end else begin
    // Tile
    XPages:=1; YPages:=1;
    if ((DR.X-UL.X)*xf>PixWidth) or ((DR.y-UL.y)*yf<-PixHeight) then begin
      Multipages:=TRUE;
      XPages:= round(((DR.X-UL.X)*xf+mmLeft*10)/PixWidth+0.5);
      YPages:= round(((DR.y-UL.y)*yf+mmTop*10)/(-PixHeight)+0.5);
    end else Multipages:=FALSE;
  end;

  PREVPageNr.MaxValue:=XPages*YPages;
  if XPages*YPages=1 then begin
    PREVPageNr.MaxValue:=1;
    CurrentPage:=1;
    PREVPageNr.Value:=CurrentPage;
    PREVPageNr.Enabled:=FALSE;
  end else PREVPageNr.Enabled:=TRUE;

  PgNrLBL.Caption:='/ '+IntToStr(XPages*YPages);
end;

procedure TPreviewForm.UpdateScaleField;
var S:string;
begin
  if PagePrinter then EXIT;
  Str(10*1/xf:3:2, S);
  PREVFactorEdit.Text:=TRIM(S);
end;

procedure TPreviewForm.PPVAllClick(Sender: TObject);
begin
  if PagePrinter then EXIT;
  PrepShowAll;
  Invalidate;
end;

procedure TPreviewForm.PREVFactorEditExit(Sender: TObject);
var c:integer;
    R:DOUBLE;
    S:string;
begin
  if PagePrinter then EXIT;
  TEdit(Sender).Text:=TRIM(TEdit(Sender).Text);
  VAL(TEdit(Sender).Text, R, c);
  if R<0.1 then R:=0.1;
  Str(R:1:2, S);
  TEdit(Sender).Text:=S;

  if c>0 then begin
    TEdit(Sender).SelStart:=0;
    TEdit(Sender).SelLength:=Length(TEdit(Sender).Text);
    TEdit(Sender).SetFocus;
  end else begin
    xf:=10*1/R;
    yf:=-xf;
  end;
  CheckRange;
  Center;
  Invalidate;
end;

procedure TPreviewForm.PREVPageNrChange(Sender: TObject);
begin
  CurrentPage:=PREVPageNr.Value;
  Invalidate;
end;

procedure TPreviewForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var Changed: boolean;
begin
  Changed:=FALSE;
  if Key=VK_Home then begin
    CurrentPage:=1;
    Changed:=TRUE;
  end;
  if Key=VK_end then begin
    CurrentPage:=XPages*YPages;
    Changed:=TRUE;
  end;

  if (Key=Ord('P')) or (Key=Ord('P')) then begin
    PREVPGNrs.Click;
    Key:=0;
  end;

  if Key=VK_Next then begin
    if Multipages then begin
      Inc(CurrentPage);
      Changed:=TRUE;
    end;
  end;
  if Key=VK_Prior then begin
    if Multipages then begin
      DEC(CurrentPage);
      Changed:=TRUE;
    end;
  end;
  if Changed then begin
    if CurrentPage<=0 then CurrentPage:=XPages*YPages;
    if CurrentPage>XPages*YPages then CurrentPage:=1;
    PREVPageNr.Value:=CurrentPage;
    Invalidate;
  end;
end;

destructor TPreviewForm.Free;
begin
end;

procedure TPreviewForm.FormDestroy(Sender: TObject);
begin
  releaseDC(handle, blatt.handle);
  panelx.Free;
  blatt.Free;
end;

procedure TPreviewForm.PREVPGNrsClick(Sender: TObject);
begin
  Invalidate;
end;

procedure TPreviewForm.PREVFactorEditChange(Sender: TObject);
var S:string;
    I:integer;
begin
  if PagePrinter then EXIT;
  S:=PREVFactorEdit.Text;
  I:=1;
  while I<=Length(S) do
    if not (S[I] in ['0'..'9', '.']) then Delete(S, I, 1) else Inc(I);
  PREVFactorEdit.Text:=S;
end;

procedure TPreviewForm.PREVFactorEditKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if PagePrinter then EXIT;
  if Key=VK_RETURN then begin
    PREVFactorEditExit(Sender);
    Key:=0;
  end;
end;

procedure TPreviewForm.PREVFactorEditKeyPress(Sender: TObject;
  var Key: Char);
begin
  if PagePrinter then EXIT;
  if Key=#13 then Key:=#0; // I don't want a beep
end;

procedure TPreviewForm.FormActivate(Sender: TObject);
begin
  if FirstActive then begin
    FirstActive:=FALSE;
    if Assigned(GlobalAppTranslator) then GlobalAppTranslator.TranslateMDI(self);
    Caption:=TransStr('[PrintPreviewCap]');
  end;
  PREVFactorEdit.Visible:=not PagePrinter;
  PPVLabel1.Visible:=not PagePrinter;
  PPVAll.Visible:=not PagePrinter;
  TL.Visible:=not PagePrinter;
  TR.Visible:=not PagePrinter;
  BL.Visible:=not PagePrinter;
  BR.Visible:=not PagePrinter;
end;

procedure TPreviewForm.PPVPrintClick(Sender: TObject);
var I:integer;
    TempCursor:integer;
begin
  TempCursor:=Screen.Cursor;
  Screen.Cursor:=crHourglass;
  if not PPVPrintMulti.Down then begin
   printer.Title:=Title;
   printer.begindoc;
   SetMapMode(printer.Canvas.handle,mm_LOMETRIC);
   ausgabe(TRUE, CurrentPage);
   printer.enddoc;
  end else begin
    try
      ProgressBar1.Visible:=TRUE;
      AbortBTN.Visible:=TRUE;
      DisableOptions(TRUE);
      printer.Title:=Title;
      printer.begindoc;
      SetMapMode(printer.Canvas.handle, mm_LOMETRIC);
      for I:=1 to XPages*YPages do begin
        ProgressBar1.Position:=round(100*I/(XPages*YPages));
				CurrentPage:=I;
        if printer.Printing and GlobalPrintShowPP then begin
          PREVPageNr.Value:=CurrentPage;
          Invalidate;
        end;
        SetMapMode(printer.Canvas.handle, mm_LOMETRIC);
        if printer.Printing then ausgabe(TRUE, I);
        if (I<XPages*YPages) and printer.Printing then printer.NewPage;
        Application.ProcessMessages;
      end;
      if printer.Printing then  printer.enddoc;
    finally
      DisableOptions(FALSE);
      ProgressBar1.Visible:=FALSE;
      AbortBTN.Visible:=FALSE;
    end;
  end;
  Screen.Cursor:=TempCursor;
end;

procedure TPreviewForm.DisableOptions(X:boolean);
begin
  X:=not X;
  PPVPrintMulti.Enabled:=X;
  PPVPrint.Enabled:=X;
  PPVSelect.Enabled:=X;
  PPVAll.Enabled:=X;
  PPVOptions.Enabled:=X;
  PREVPGNrs.Enabled:=X;
  PREVPageNr.Enabled:=X;
  StopPreview.Enabled:=X;
  PREVFactorEdit.Enabled:=X;
end;

procedure TPreviewForm.AbortBTNClick(Sender: TObject);
begin
  if printer.Printing then printer.Abort;
end;

procedure TPreviewForm.PPVOptionsClick(Sender: TObject);
begin
  OptionsDLG.PageControl1.ActivePage:=OptionsDLG.PrintTab;
  OptionsDLG.ShowModal;
  if Assigned(GlobalAppTranslator) then GlobalAppTranslator.TranslateMDI(self);
  Caption:=TransStr('[PrintPreviewCap]');
  Invalidate;
end;

procedure TPreviewForm.StopPreviewClick(Sender: TObject);
begin
  Close;
end;

procedure TPreviewForm.BRClick(Sender: TObject);
begin
  InfoArrange:=0;
  if TL.Down then InfoArrange:=1;
  if TR.Down then InfoArrange:=2;
  if BL.Down then InfoArrange:=3;
  if BR.Down then InfoArrange:=4;
  Invalidate;
end;

end.
