(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit RailyTypes;

interface

USES ImportTypes, Classes, ExtCtrls, ComCtrls, AppTranslator;

type
    TElementInfo = PACKED RECORD
                      Name           : array[0..20] of CHAR;
                      Remark         : array[0..50] of CHAR;
                      TheType        : TImportSchienenTyp;
                      Uebergang      : array[0..25] of CHAR;
                      TheRadius      : Single;
                      TheLength      : Single;
                      TheAngle       : Single;
                    end;

    TModuleInfo  = PACKED RECORD
                      Description    : array[0..128] of CHAR;
                      Signature      : array[0..9]  of CHAR;
                      Name           : array[0..25] of CHAR;
                      System         : array[0..4]  of CHAR;
                      Manufacturer   : array[0..32] of CHAR;
                      Author         : array[0..32] of CHAR;
                      Remark         : array[0..128]of CHAR;
                      Spurweite      : Double;
                      ElWidth        : Double;
                    end;

    PShortModuleInfo = ^TShortModuleInfo;
    TShortModuleInfo = PACKED RECORD
                      System         : array[0..4]  of CHAR;
                      Manufacturer   : array[0..32] of CHAR;
                      Spurweite      : Double;
                      FileName       : string;
                    end;

    TPlanInfo  = PACKED RECORD
                      Description    : array[0..128] of CHAR;
                      Signature      : array[0..9]  of CHAR;
                      Author         : array[0..32] of CHAR;
                      Remark         : array[0..128]of CHAR;
                    end;

    TRealPoint = RECORD
      X, Y:real;
    end;

    TDoublePoint = RECORD
      X, Y:Double;
    end;

    TSinglePoint = RECORD
      X, Y: Single;
    end;

    PConnector = ^TConnector;
    TConnector = PACKED RECORD
                   Nr     : longint;
                   Height : Single;
                   Electro: longint;
                 end;

    TConnectorListPoint = class (TObject)
      P         : TDoublePoint;
      A         : Double;
      ID        : integer;
      Cnct      : integer;
      TempAngle : Double;
    end;

    TUserTextEntry = class (TObject)
      P         : TDoublePoint;
      S         : string;
    end;

    TTextListPoint = class (TObject)
      A, B      : TDoublePoint;
      RailText  : boolean;
      ID        : integer;
    end;


    TPointObject = class (TObject)
      P         : TDoublePoint;
      constructor Create(X,Y:Double);
    end;

    PDrawInfo = ^TDrawInfo;
    TDrawInfo = RECORD
      Spurweite         : Double;
      ElWidth           : Double;
      ClickBreite       : Double;
      DoubleDraw        : BYTE;
      CnctDraw          : BYTE;
      Exact             : boolean;
      Precision         : longint; // 1/delta_rad
      OverruleColor     : boolean;
      TextColorOverrule : boolean;
      Blacked           : boolean;
      ShowRailNames     : boolean;
      Selected          : boolean;
      ForceTransparent  : boolean;
    end;

    TParentElements = RECORD
      Overview      : TPaintbox;
      Preview       : TPaintbox;
      Timer         : TTimer;
      Status        : TStatusbar;
{      ChooseBTN     : procedure (X:TImportSchienenTyp) of object;
      ChooseMode    : procedure (X:integer) of object;
      SpecialMode   : procedure (X:boolean) of object;
      IsClosing     : procedure of object;
      InfoFloaterVisible: procedure (X:boolean) of object;}
      Translator    : TAppTranslator;
    end;

    type TMyString = class(TObject)
        public
          X : string;
          constructor Create(T:string);
     end;


implementation

constructor TMyString.Create(T:string);
begin
  X:=T;
end;

constructor TPointObject.Create(X,Y:Double);
begin
  P.X:=X;P.Y:=Y;
end;

end.
