(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit Scanner;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ToolWin, ComCtrls, Buttons, StdCtrls, checklst, ExtCtrls, Plan, Instances, Globals;

type
  TScannerForm = class(TForm)
    Timer1: TTimer;
    Panel2: TPanel;
    PaintBox1: TPaintBox;
    Panel3: TPanel;
    AbortSearchBTN: TBitBtn;
    SearchBTN: TBitBtn;
    SearchBaseEdit: TEdit;
    Label3: TLabel;
    Label2: TLabel;
    DirCountLBL: TLabel;
    FileCountLBL: TLabel;
    Label1: TLabel;
    CurrentPath: TLabel;
    FileListBox: TListBox;
    Button1: TButton;
    BitBtn1: TBitBtn;
    ConditionBOX: TComboBox;
    Label4: TLabel;
    CheckBox1: TCheckBox;
    InfoPanel: TPanel;
    ListBox1: TListBox;
    Label5: TLabel;
    Label6: TLabel;
    NumElements: TLabel;
    RemarkMemo: TMemo;
    Label7: TLabel;
    TrackLength: TLabel;
    StayOnTop: TCheckBox;
    Label8: TLabel;
    RealTrackLength: TLabel;
    procedure SearchBTNClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure AbortSearchBTNClick(Sender: TObject);
    procedure FileListBoxClick(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure FileListBoxDblClick(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure HideBrowserClick(Sender: TObject);
    procedure StayOnTopClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { private-Deklarationen }
    SearchAborted:boolean;
    DirCount,
    FileCount:integer;
    MyPlan:TPlan;
    procedure CalcModulesInUse(X:TPlan; S:TStringList);
  public
    { public-Deklarationen }
  end;

var
  ScannerForm: TScannerForm;

implementation

uses SelectDirDLG;

{$R *.DFM}

procedure TScannerForm.SearchBTNClick(Sender: TObject);
var Condition:string;
{    bi:TBorderIcons;}

  procedure SearchRecursive(Where:string);
  var SearchRec:TSearchRec;
      Found: integer;
  begin
    Inc(DirCount);
    CurrentPath.Caption:=Where;
    Found := FindFirst(Where+'*.RLY', (faReadOnly + faHidden), SearchRec);
    while (Found = 0) and not SearchAborted do begin
      if (Condition='') or (POS(Condition, UpperCase(Where+SearchRec.name))>0) then begin
        FileListBox.items.Add(Where+SearchRec.name);
        Inc(FileCount);
      end;
      Found := FindNext(SearchRec);
    end;
    Application.ProcessMessages;
    FindClose(SearchRec);

    Found := FindFirst(Where+'*.*', (faDirectory), SearchRec);
    while (Found = 0) and not SearchAborted do begin
        Application.ProcessMessages;
        if (SearchRec.Attr and faDirectory) > 0 then begin
          if (SearchRec.name<>'.') and (SearchRec.name<>'..')
                                   then SearchRecursive(Where+SearchRec.name+'\');
        end;
        Found := FindNext(SearchRec);
    end;
    FindClose(SearchRec);
  end;
begin
  if Length(SearchBaseEdit.Text)=0 then Exit;
  SearchBTN.Enabled:=FALSE;
  FileCount:=0;  DirCount:=0;  FileListBox.Clear;
  AbortSearchBTN.Visible:=TRUE;
{  bi:=BorderIcons;
  BorderIcons:=[];}
  Timer1.Enabled:=TRUE;
  SearchAborted:=FALSE;

  Condition:=UpperCase(ConditionBOX.Text);
  if Condition='<ALLE>' then Condition:='';
  if SearchBaseEdit.Text[Length(SearchBaseEdit.Text)]<>'\' then SearchBaseEdit.Text:=SearchBaseEdit.Text+'\';
  SearchRecursive(SearchBaseEdit.Text);
  if not SearchAborted then CurrentPath.Caption:='Search finished.'
                       else CurrentPath.Caption:='Search aborted.';

  Timer1Timer(Self);
  Timer1.Enabled:=FALSE;
  AbortSearchBTN.Visible:=FALSE;
  SearchBTN.Enabled:=TRUE;
{  BorderIcons:=bi;}
end;

procedure TScannerForm.Timer1Timer(Sender: TObject);
begin
  DirCountLBL.Caption:=IntToStr(DirCount);
  FileCountLBL.Caption:=IntToStr(FileCount);
end;

procedure TScannerForm.AbortSearchBTNClick(Sender: TObject);
begin
  SearchAborted:=TRUE;
end;

procedure TScannerForm.CalcModulesInUse(X:TPlan; S:TStringList);
var i:integer;
    TempInst:TInstance;
begin
  S.Clear;
  S.Sorted:=TRUE;
  S.Duplicates:=dupIgnore;
  WITH X do for i:=0 TO InstanceElements.Count-1 do
    if GetElement(i, TempInst) and Assigned(TempInst) then
      S.Add(StrPas(TempInst.ModuleName));
end;

procedure TScannerForm.FileListBoxClick(Sender: TObject);
var S:TStringList;
    St:string;
begin
  MyPlan.Free; MyPlan:=TPlan.Create;
  if (not MyPlan.Load(FileListBox.items[FileListBox.ItemIndex])) and
  (not MyPlan.ImportWR1(FileListBox.items[FileListBox.ItemIndex])) then begin
    Panel2.Caption:='Module Error';
    NumElements.Caption:='';
    RemarkMemo.SetTextBuf('');
    TrackLength.Caption:='';
    RealTrackLength.Caption:='';
  end else begin
    Panel2.Caption:='';

    Str(MyPlan.CalcLength/1000:0:2, St);
    TrackLength.Caption:=St+'m';

    Str(MyPlan.CalcRealLength/(1000*1000):0:2, St);
    RealTrackLength.Caption:=St+'km';

    S:=TStringList.Create;
    CalcModulesInUse(MyPlan, S);
    ListBox1.Clear; ListBox1.items.AddStrings(S);
    S.Free;
    NumElements.Caption:=IntToStr(MyPlan.InstanceElements.Count);
    RemarkMemo.SetTextBuf(MyPlan.Info.Remark);
  end;
  PaintBox1.Invalidate;
end;

procedure TScannerForm.PaintBox1Paint(Sender: TObject);
var xs,ys, Overviewxs, Overviewys, Overviewf:Double;
begin
  if not Assigned(MyPlan) then Exit;
    MyPlan.CalcRectAround;
    WITH MyPlan do begin
      try
        xs:=1; ys:=1;
        if ABS(LR.X-UL.X)>0 then xs:=(PaintBox1.Width-6)/ABS(LR.X-UL.X);
        if ABS(LR.Y-UL.Y)>0 then ys:=(PaintBox1.Height-6)/ABS(LR.Y-UL.Y);
        if xs<ys then Overviewf:=xs else Overviewf:=ys;
        Overviewxs:=3/Overviewf-UL.X+((PaintBox1.Width-6-(LR.X-UL.X)*Overviewf)/2)/Overviewf;
        Overviewys:=3/Overviewf-UL.Y+((PaintBox1.Height-6-(LR.Y-UL.Y)*Overviewf)/2)/Overviewf;
        Draw(PaintBox1.Canvas, Overviewxs, Overviewys, Overviewf, Overviewf, TRUE, TRUE, -1);
        MyPlan.DrawTable(PaintBox1.Canvas, Overviewxs, Overviewys, Overviewf, Overviewf, -1, FALSE);
      except end;
    end;
end;

procedure TScannerForm.FormCreate(Sender: TObject);
begin
  MyPlan:=TPlan.Create;
  SearchBaseEdit.Text:=GlobalWorkPath;
end;

procedure TScannerForm.FormDestroy(Sender: TObject);
begin
  MyPlan.Free;
end;

procedure TScannerForm.FormResize(Sender: TObject);
var t:integer;
begin
  t:=Height-Panel3.Height-FileListBox.Height-30;
  if InfoPanel.Visible then t:=t-InfoPanel.Height;
  Panel2.Height:=t;
end;


procedure TScannerForm.Button1Click(Sender: TObject);
begin
  SelectDirForm.Caption:='Start-Verzeichnis fr die Suche auswhlen';
  if SelectDirForm.ShowModal =idOK then SearchBaseEdit.Text:=SelectDirForm.Label1.Caption;
end;

procedure TScannerForm.BitBtn1Click(Sender: TObject);
var hw:HWND;
    At:ATOM;
    i:integer;
    St:string;
begin
  hw:=FindWindow('ENIGON Raily 2.1', NIL);
  if hw=0 then Exit;
  for i:=0 TO FileListBox.items.Count-1 do
    if FileListBox.Selected[i] then begin
     St:=FileListBox.items[i];
     At:=GlobalAddAtom(PChar(St));
     SendMessage(hw, WM_LoadFile, Length(St), integer(At));
  end;
end;

procedure TScannerForm.FileListBoxDblClick(Sender: TObject);
var hw:HWND;
    At:ATOM;
    St:string;
begin
  hw:=FindWindow('ENIGON Raily 2.1', NIL);
  if hw=0 then Exit;
  try
    St:=FileListBox.items[FileListBox.ItemIndex];
    At:=GlobalAddAtom(PChar(St));
    SendMessage(hw, WM_LoadFile, Length(St), integer(At));
  except end;
end;

procedure TScannerForm.CheckBox1Click(Sender: TObject);
begin
  InfoPanel.Visible:=CheckBox1.Checked;
  FormResize(Self);
end;

procedure TScannerForm.HideBrowserClick(Sender: TObject);
begin
  Hide;
end;

procedure TScannerForm.StayOnTopClick(Sender: TObject);
begin
  if StayOnTop.Checked then FormStyle:=fsStayOnTop else FormStyle:=fsNormal;
end;

procedure TScannerForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if AbortSearchBTN.Visible then Action:=caNone;
end;

end.

