(* Copyright (c) 1994-2001, Enigon Software
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list
  of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this
  list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.
- Neither the name of Enigon Software nor the names of its contributors may be
  used to endorse or promote products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- end of licence --
Author: tfrey@enigon.com
Compiled version of Raily 2.0 can be found at http://www.enigon.com/museum *)
unit UInventory;

interface
 USES Classes, SysUtils;

 type TInventoryListEntry = class (TObject)
        Module    : string;
        Element   : string;
        Count     : integer;
        Has       : integer;
        Length    : Double;
        constructor Create(M, E:string; C, H:integer);
        procedure OneMore;
      end;

     TInventory = class (TObject)
        List    : TList;
        constructor Create;
        procedure Reset;
        procedure ResetCount;
        procedure Enter(M, E: string);
        procedure Know(M, E:string);
        procedure Has(M, E:string; X:integer);
        procedure IncLength(M, E:string; L:Double);
        procedure Sort;
        function Find(M, E:string; var Nr:integer):boolean;
        destructor Free;
     end;

implementation

constructor TInventoryListEntry.Create(M, E:string; C, H:integer);
begin
	Module:=M; Element:=E; Count:=C; Has:=H; Length:=0
end;

procedure TInventoryListEntry.OneMore;
begin
	INC(Count)
end;

constructor TInventory.Create;
begin
	List := TList.Create
end;

procedure TInventory.Reset;
var i: integer;
begin
	for i:=List.Count-1 downto 0 do begin
		TInventoryListEntry(List[i]).Free;
		List.Delete(i)
	end
end;

procedure TInventory.ResetCount;
var i:integer;
begin
	for i:=List.Count-1 downto 0 do begin
		TInventoryListEntry(List[i]).Count:=0;
		TInventoryListEntry(List[i]).Length:=0;
		if TInventoryListEntry(List[i]).Has=0 then begin
			TInventoryListEntry(List[i]).Free;
			List.Delete(i)
		end
	end
end;

procedure TInventory.Know(M, E:string);
var Nr:integer;
begin
	if not Find(M, E, Nr) then List.Add(TInventoryListEntry.Create(M, E, 0, 0))
end;

procedure TInventory.IncLength(M, E:string; L:Double);
var Nr:integer;
begin
	Know(M, E);
	if Find(M, E, Nr) then TInventoryListEntry(List[Nr]).Length:=TInventoryListEntry(List[Nr]).Length+L
										else raise Exception.Create('Inventory failed')
end;

procedure TInventory.Enter(M, E: string);
var Nr:integer;
begin
	Know(M, E);
	if Find(M, E, Nr) then TInventoryListEntry(List[Nr]).OneMore
										else raise Exception.Create('Inventory failed')
end;

procedure TInventory.Has(M, E:string; X:integer);
var Nr:integer;
begin
	Know(M, E);
	if Find(M, E, Nr) then TInventoryListEntry(List[Nr]).Has:=X
										else raise Exception.Create('Inventory failed')
end;

function TInventory.Find(M, E:string; var Nr:integer):boolean;
var i:integer;
		T:TInventoryListEntry;
begin
	Result:=FALSE;
	for i:=0 TO List.Count-1 do begin
		T:=TInventoryListEntry(List[i]);
		if (T.Element=E) and (T.Module=M) then begin
			Nr:=i;
			Result:=True;
			Break
		end
	end
end;

	function SortModule(Item1, Item2: Pointer): integer;
	begin
		if Assigned(Item1) and Assigned(Item2) then begin
			Result:=0;
			if TInventoryListEntry(Item1).Module<TInventoryListEntry(Item2).Module then Result:=-1;
			if TInventoryListEntry(Item1).Module>TInventoryListEntry(Item2).Module then Result:=1;
			if Result=0 then begin
				if TInventoryListEntry(Item1).Element<TInventoryListEntry(Item2).Element then Result:=-1;
				if TInventoryListEntry(Item1).Element>TInventoryListEntry(Item2).Element then Result:=1
			end
		end else Result:=0
	end;

procedure TInventory.Sort;
begin
	List.Sort(SortModule)
end;

destructor TInventory.Free;
begin
	Reset;
	List.Free
end;

end.
